/* PKI_X509_REQ I/O management */

#include <libpki/pki.h>

PKI_X509_REQ *PKI_X509_REQ_get ( char *url_s ) {

	PKI_X509_REQ_STACK *r_sk = NULL;
	PKI_X509_REQ *tmp_r = NULL;
	PKI_X509_REQ *ret = NULL;

	if( !url_s ) return (NULL);

	if((r_sk = PKI_X509_REQ_STACK_get( url_s )) == NULL ) {
		return(NULL);
	}

	if( PKI_STACK_X509_REQ_elements( r_sk ) >= 1 ) {
		ret = PKI_STACK_X509_REQ_pop( r_sk );
	}

	while( (tmp_r = PKI_STACK_X509_REQ_pop( r_sk )) != NULL ) {
		PKI_X509_REQ_free ( tmp_r );
	}

	return( ret );
}

PKI_X509_REQ *PKI_X509_REQ_get_url ( URL *url ) {

	PKI_X509_REQ_STACK *r_sk = NULL;
	PKI_X509_REQ *tmp_r = NULL;
	PKI_X509_REQ *ret = NULL;

	if( !url ) return (NULL);

	if((r_sk = PKI_X509_REQ_STACK_get_url( url )) == NULL ) {
		return(NULL);
	}

	if( PKI_STACK_X509_REQ_elements( r_sk ) >= 1 ) {
		ret = PKI_STACK_X509_REQ_pop( r_sk );
	}

	while( (tmp_r = PKI_STACK_X509_REQ_pop( r_sk )) != NULL ) {
		PKI_X509_REQ_free ( tmp_r );
	}

	return (ret);
}

PKI_X509_REQ_STACK *PKI_X509_REQ_STACK_get ( char *url_s ) {

	URL *url = NULL;
	PKI_X509_REQ_STACK *ret_sk = NULL;

	if( !url_s ) return (NULL);

	if((url = URL_new(url_s)) == NULL ) {
		return (NULL);
	}

	ret_sk = PKI_X509_REQ_STACK_get_url ( url );
	
	URL_free( url );

	return( ret_sk );
}

PKI_X509_REQ_STACK *PKI_X509_REQ_STACK_get_url ( URL *url ) {

	PKI_X509_REQ_STACK * ret_sk = NULL;

	if( !url ) return (NULL);

	switch( url->proto ) {
		case URI_PROTO_FILE:
			ret_sk = PKI_X509_REQ_STACK_get_file( url );
			break;
	}

	return (ret_sk);
}

int PKI_X509_REQ_write(PKI_X509_REQ *req, int format, char *url_s) {

	PKI_X509_REQ_STACK *sk = NULL;
	int ret = PKI_OK;

	if( !req || !url_s ) return (PKI_ERR);

	if(( sk = PKI_STACK_X509_REQ_new()) == NULL ) return (PKI_ERR);

	if(PKI_STACK_X509_REQ_push( sk, req ) == PKI_ERR ) {
		PKI_STACK_X509_REQ_free( sk );
		return (PKI_ERR);
	}

	ret = PKI_X509_REQ_STACK_write( sk, format, url_s );

	if( sk ) {
		PKI_X509_REQ *tmp_r = NULL;
		while((tmp_r = PKI_STACK_X509_REQ_pop ( sk )) != NULL ) {
			PKI_STACK_X509_REQ_free (sk);
		}
	}

	return (ret);
}

int PKI_X509_REQ_write_url(PKI_X509_REQ *req, int format, URL *url) {

	PKI_X509_REQ_STACK *sk = NULL;
	int ret = PKI_OK;

	if( !req || !url ) return (PKI_ERR);

	if(( sk = PKI_STACK_X509_REQ_new()) == NULL ) return (PKI_ERR);

	if(PKI_STACK_X509_REQ_push( sk, req ) == PKI_ERR ) {
		PKI_STACK_X509_REQ_free( sk );
		return (PKI_ERR);
	}

	ret = PKI_X509_REQ_STACK_write_url( sk, format, url );
	if( sk ) {
		PKI_X509_REQ *tmp_r = NULL;
		while( ( tmp_r = PKI_STACK_X509_REQ_pop ( sk )) != NULL ) {
			PKI_STACK_X509_REQ_free (sk);
		}
	}

	return( ret );
}

int PKI_X509_REQ_STACK_write(PKI_X509_REQ_STACK *sk, int format, char *url_s) {

	int ret = PKI_OK;
	URL *url = NULL;

	if(!sk || !url_s ) return( PKI_ERR );

	if(PKI_STACK_X509_REQ_elements(sk) < 1 ) return (PKI_ERR);

	if((url = URL_new(url_s)) == NULL ) {
		return(PKI_ERR);
	}

	ret = PKI_X509_REQ_STACK_write_url( sk, format, url );

	URL_free ( url );

	return (ret);

}

int PKI_X509_REQ_STACK_write_url(PKI_X509_REQ_STACK *sk, int format, URL *url) {

	int ret = PKI_OK;

	if( !sk || !url ) return (PKI_ERR);

	switch( url->proto ) {
		case URI_PROTO_FILE:
			ret = PKI_X509_REQ_STACK_write_file( sk, format, url);
			break;
	}

	return ( ret );
}


/*
 * X509_REQ File Operations
 */

PKI_X509_REQ_STACK *PKI_X509_REQ_STACK_get_file ( URL *url ) {

        PKI_X509_REQ *req= NULL;
        BIO *in = NULL;
        PKI_X509_REQ_STACK *r_sk = NULL;

	/* Check the pointer */
	if( !url || !url->addr ) return NULL;

	/* Open the file in addr */
        if ((in=BIO_new_file( url->addr, "r")) == NULL) {
                return(NULL);
        }

	/* Initialize the structure */
        if((r_sk = PKI_STACK_X509_REQ_new()) == NULL ) {
                return(NULL);
        }

	/* Try PEM FORMAT */
	while(( req = (PKI_X509_REQ *) PEM_read_bio_X509_REQ(in, 
					NULL, NULL, NULL)) == NULL ) {
		PKI_STACK_X509_REQ_push(r_sk, req);
	}

	if( PKI_STACK_X509_REQ_elements( r_sk ) < 1 ) {
		/* No PEM formatted files have been found */
		while(( req = (PKI_X509_REQ *) d2i_X509_bio(in, NULL)) 
								== NULL ) {
			/* DER formatted Certificate found */
			PKI_STACK_X509_REQ_push(r_sk, req);
		}
	}

	if( PKI_STACK_X509_REQ_elements(r_sk) == 0 ) {
		PKI_STACK_X509_REQ_free (r_sk );
		return NULL;
	}
        return r_sk;
}

int PKI_X509_REQ_STACK_write_file( PKI_X509_REQ_STACK *sk, int format, 
								URL *url ) {
	BIO *out = NULL;
	PKI_X509_REQ *req = NULL;

	int ret = PKI_OK;
	int i = 0;

	if( !sk ) return (PKI_ERR);

	if((out = BIO_new(BIO_s_file())) == NULL ) {
		/* Memory error (?) */
		fprintf(stderr, "%s:%d error\n", __FILE__, __LINE__ );
		return (PKI_ERR);
	}

	if( !url || !url->addr ) {
		/* standard output selected */
		 ret = BIO_set_fp(out,stdout,BIO_NOCLOSE);
	} else {
		ret = (int ) BIO_write_filename( out, url->addr);
	}

	if( ret == 0 ) {
		BIO_free_all( out );
		return( PKI_ERR );
	}

	ret = PKI_OK;

	for( i=0; i < PKI_STACK_X509_REQ_elements( sk ); i++ ) {
		if((req = PKI_STACK_X509_REQ_get_num( sk, i )) == NULL) {
			break;
		}

		switch( format ) {
			case PKI_FORMAT_PEM:
				ret = PEM_write_bio_X509_REQ( out, 
							(X509_REQ *)req);
				break;
			case PKI_FORMAT_ASN1:
				ret = i2d_X509_REQ_bio( out, (X509_REQ *) req);
			default:
				/* Format not recognized ! */
				printf("%s:%d error\n", __FILE__, __LINE__ );
				ret = PKI_ERR;
				break;
		}
	}

	if( out ) BIO_free_all (out);

	return(ret);
}

