/*
 * OpenCA Project
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 1999
 *
 * Copyright (c) 2001 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include <libpki/pki.h>

PKI_MEM_STACK *URL_get_data ( char *url_s, ssize_t size ) {

	URL *url = NULL;

	if( !url_s ) return (NULL);
	if(( url = URL_new(url_s)) == NULL ) {
		return (NULL);
	}

	return( URL_get_data_url( url, size ));
}

PKI_MEM_STACK *URL_get_data_file( URL *url, ssize_t size ) {

	PKI_MEM_STACK * ret = NULL;
	PKI_MEM * obj = NULL;
	ssize_t file_size = 0;
	int fd = 0;

	if( !url ) return (NULL);
	if( url->proto != URI_PROTO_FILE ) return (NULL);

	if((fd = open( url->addr, O_RDONLY)) == -1 ) {
		return (NULL);
	}

	if( size == 0 ) size = LONG_MAX - 1;

	if((ret = PKI_STACK_MEM_new()) == NULL ) {
		return( NULL );
	}
	if((obj = PKI_MEM_new_null()) == NULL ) {
		PKI_STACK_MEM_free(ret);
		return ( NULL );
	}

	lseek( fd, 0, SEEK_END);
	file_size = lseek( fd, 0, SEEK_CUR);

	if( file_size > size ) file_size = size;

	lseek( fd, 0, SEEK_SET);

	PKI_MEM_grow( obj, file_size );
	if((read( fd, obj->data, file_size)) == -1 ) {
		/* Error ?!?!? */
		PKI_MEM_free( obj );
		PKI_STACK_MEM_free ( ret );
		ret = NULL;
	} else {
		obj->size = file_size;
	}
	close( fd );

	PKI_STACK_MEM_push( ret, obj );

	return( ret );
}


PKI_MEM_STACK *URL_get_data_url ( URL *url, ssize_t size ) {


	PKI_MEM_STACK * ret = NULL;

	if( !url ) {
		return ( NULL );
	}

	switch( url->proto ) {
		case URI_PROTO_FILE:
			ret = URL_get_data_file( url, size );
			break;
		case URI_PROTO_HTTP:
			ret = URL_get_data_http_url( url, size );
			break;
		case URI_PROTO_LDAP:
			ret = URL_get_data_ldap_url( url, size );
			break;
		case URI_PROTO_MYSQL:
			ret = URL_get_data_mysql_url( url, size );
			break;
		case URI_PROTO_PG:
			ret = URL_get_data_pg_url( url, size );
			break;
		case URI_PROTO_PKCS11:
			ret = URL_get_data_pkcs11_url( url, size );
			break;
		case URI_PROTO_FTP:
		default:
			ret = NULL;
	}

	return ( ret );
}

int URL_put_data ( char *url_s, PKI_MEM *data, char *contType ) {

	URL *url = NULL;
	int ret = 0;

	if( !url_s || !data ) {
		return (PKI_ERR);
	}

	if(( url = URL_new(url_s)) == NULL ) {
		return (PKI_ERR);
	}

	ret = URL_put_data_url( url, data, contType );

	if( url ) URL_free ( url );

	return( ret );
}

int URL_put_data_file ( URL *url, PKI_MEM *data ) {

	int fd = 0;

	if( !url || !data || !url->addr ) return ( PKI_ERR );

	if(( fd = open ( url->addr, O_RDWR, S_IRUSR | S_IWUSR )) == -1 ) {
		return ( PKI_ERR );
	}

	if(_Write( fd, data->data, data->size ) < 0 ) {
		close (fd);
		return ( PKI_ERR );
	}

	close ( fd );

	return ( PKI_OK );
	
}

int URL_put_data_url ( URL *url, PKI_MEM *data, char *contType ) {


	int ret = PKI_OK;

	if( !url || !data ) {
		return ( PKI_ERR );
	}

	switch( url->proto ) {
		case URI_PROTO_FILE:
			ret = URL_put_data_file( url, data );
			break;
		// case URI_PROTO_HTTP:
		// 	ret = URL_put_data_http_url( url, contType );
		// 	break;
		// case URI_PROTO_LDAP:
		// 	ret = URL_put_data_ldap_url( url );
		// 	break;
		case URI_PROTO_MYSQL:
			ret = URL_put_data_mysql_url( url, data );
			break;
		case URI_PROTO_PG:
			ret = URL_put_data_pg_url( url, data );
			break;
		// case URI_PROTO_PKCS11:
		// 	ret = URL_put_data_pkcs11_url( url, size );
		// 	break;
		case URI_PROTO_FTP:
		default:
			ret = PKI_ERR;
	}

	return ( ret );
}

URL *URL_new ( char *url_s ) {

	URL *ret = NULL;
	char *tmp_s = NULL;
	char *tmp_s2 = NULL;
	char *tmp_s3 = NULL;
	char *tmp_s4 = NULL;
	int len;

	if( !url_s ) return NULL;

	ret = (URL *) malloc ( sizeof( URL ));
	if( !ret ) return NULL;

	memset( ret, 0, sizeof(struct url_data));
	ret->addr  = NULL;
	ret->pwd   = NULL;
	ret->usr   = NULL;
	ret->url_s = NULL;
	ret->attrs = NULL;
	ret->port  = 0;

	if((ret->url_s = (char *) PKI_Malloc ( strlen( url_s ) + 1)) == NULL) {
		URL_free (ret);
		return(NULL);
	}

	strncpy( ret->url_s, url_s, strlen(url_s));

	if( strncmp("ldap://", url_s, 6 ) == 0) {
		ret->proto = URI_PROTO_LDAP;
		ret->port = DEFAULT_LDAP_PORT;

		tmp_s = &url_s[7];

		if((tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s4 = strchr(tmp_s, '/');
			if( tmp_s4 && tmp_s3 < tmp_s4 ) {
				tmp_s2 = strchr( tmp_s, ':' );

				if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
					return NULL;
				}
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->usr = (char *) malloc (len+1);
				memset( ret->usr, 0, len+1);
				strncpy( ret->usr, tmp_s, len);
				ret->usr[len] = '\x0';
			
				tmp_s = tmp_s2+1;
				tmp_s2 = strchr( tmp_s,'@');

				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->pwd = (char *) malloc (len+1);
				memset( ret->pwd, 0, len+1);
				strncpy( ret->pwd, tmp_s, len);
				ret->pwd[len] = '\x0';

				tmp_s = tmp_s2+1;
			}
		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset( ret->addr, 0, len+1 );

			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
			if( ret->port == 0 ) {
				/* Error in parsing the port number! */
				URL_free ( ret );
				return(NULL);
			}

			if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
				tmp_s =  tmp_s2;
			} else {
				tmp_s = &url_s[7];
			}
		}

		if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
			char *tmp_attrs = NULL;
			tmp_s2++;

			if((tmp_attrs = strchr( tmp_s, '?' )) != NULL) {
				len = strlen( tmp_attrs );
				ret->attrs = (char *) malloc ( len );
				memset( ret->attrs, 0, len );
				memcpy( ret->attrs, tmp_attrs + 1, len);
				len = tmp_attrs - tmp_s2;

				ret->path = (char *) malloc ( len +1);
				memset( ret->path, 0, len +1);
				memcpy( ret->path, tmp_s2, len);

				if( !ret->addr ) {
					len = (int) (tmp_s2 - tmp_s - 1);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}
			} else {
				if( !ret->addr ) {
					len = (int) ( (long) tmp_s2 - (long) tmp_s );
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}

				tmp_s = tmp_s2;
				ret->path = strdup( tmp_s );
			}
		} else {
			if( !ret->addr )
				ret->addr = strdup( tmp_s );
			ret->path = strdup("/");
		}

	} else if( strncmp("mysql://", url_s, 8 ) == 0) {
		ret->proto = URI_PROTO_MYSQL;
		ret->port = DEFAULT_MYSQL_PORT;

		tmp_s = &url_s[8];

		if((tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s4 = strchr(tmp_s, '/');
			if( tmp_s4 && tmp_s3 < tmp_s4 ) {
				tmp_s2 = strchr( tmp_s, ':' );

				if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
					return (NULL);
				}
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->usr = (char *) malloc (len+1);
				memset( ret->usr, 0, len+1);
				strncpy( ret->usr, tmp_s, len);
				ret->usr[len] = '\x0';
			
				tmp_s = tmp_s2+1;
				tmp_s2 = strchr( tmp_s,'@');

				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->pwd = (char *) malloc (len+1);
				memset( ret->pwd, 0, len+1);
				strncpy( ret->pwd, tmp_s, len);
				ret->pwd[len] = '\x0';

				tmp_s = tmp_s2+1;
			}
		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset( ret->addr, 0, len+1 );

			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
			if( ret->port == 0 ) {
				/* Error in parsing the port number! */
				URL_free ( ret );
				return(NULL);
			}

			if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
				tmp_s =  tmp_s2;
			} else {
				tmp_s = &url_s[7];
			}
		}

		if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
			char *tmp_attrs = NULL;
			tmp_s2++;

			if((tmp_attrs = strchr( tmp_s, '?' )) != NULL) {
				len = strlen( tmp_attrs );
				ret->attrs = (char *) malloc ( len );
				memset( ret->attrs, 0, len );
				memcpy( ret->attrs, tmp_attrs + 1, len);
				/*
				while( ((tmp_s3 = 
					strchr(tmp_s2,'(')) != NULL) && 
							(tmp_s3 < tmp_s4) ) {
					char *tmp_s5 = NULL;
					char *tmp_s6 = NULL;

					if( (tmp_s5 = strchr(tmp_s3, ')')) 
								== NULL) {
						URL_free( ret );
						return(NULL);
					}
					tmp_s3++;
					tmp_s6 = strchr(tmp_s3, '=');
					if( tmp_s6 > tmp_s5 ) {
						URL_free( ret );
						return(NULL);
					}
					tmp_s6 = (char *) malloc ( tmp_s5 - tmp_s3+1);
					memset( tmp_s6, 0, tmp_s5 - tmp_s3 + 1);
					memcpy( tmp_s6, tmp_s3, tmp_s5 - tmp_s3);
					PKI_STACK_push( ret->attrs, tmp_s6 );

					tmp_s2 = tmp_s3;
				}
				tmp_s = tmp_s4;
				*/
				len = tmp_attrs - tmp_s2;
				ret->path = (char *) malloc ( len );
				memset( ret->path, 0, len);
				memcpy( ret->path, tmp_s2, len - 1);

				if( !ret->addr ) {
					len = (int) (tmp_s2 - tmp_s - 1);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}
			} else {
				if( !ret->addr ) {
					len = (int) 
						((long) tmp_s2 - (long) tmp_s);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}

				tmp_s = tmp_s2;
				ret->path = strdup( tmp_s );
			}
		} else {
			if( !ret->addr )
				ret->addr = strdup( tmp_s );
			ret->path = strdup("/");
		}
	} else if( strncmp("pg://", url_s, 5 ) == 0) {
		ret->proto = URI_PROTO_PG;
		ret->port = DEFAULT_PG_PORT;

		tmp_s = &url_s[5];

		if((tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s4 = strchr(tmp_s, '/');
			if( tmp_s4 && tmp_s3 < tmp_s4 ) {
				tmp_s2 = strchr( tmp_s, ':' );

				if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
					return (NULL);
				}
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->usr = (char *) malloc (len+1);
				memset( ret->usr, 0, len+1);
				strncpy( ret->usr, tmp_s, len);
				ret->usr[len] = '\x0';
			
				tmp_s = tmp_s2+1;
				tmp_s2 = strchr( tmp_s,'@');

				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->pwd = (char *) malloc (len+1);
				memset( ret->pwd, 0, len+1);
				strncpy( ret->pwd, tmp_s, len);
				ret->pwd[len] = '\x0';

				tmp_s = tmp_s2+1;
			}
		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset( ret->addr, 0, len+1 );

			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
			if( ret->port == 0 ) {
				/* Error in parsing the port number! */
				URL_free ( ret );
				return(NULL);
			}

			if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
				tmp_s =  tmp_s2;
			} else {
				tmp_s = &url_s[7];
			}
		}

		if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
			char *tmp_attrs = NULL;
			tmp_s2++;

			if((tmp_attrs = strchr( tmp_s, '?' )) != NULL) {
				len = strlen( tmp_attrs );
				ret->attrs = (char *) malloc ( len );
				memset( ret->attrs, 0, len );
				memcpy( ret->attrs, tmp_attrs + 1, len);
				/*
				while( ((tmp_s3 = 
					strchr(tmp_s2,'(')) != NULL) && 
							(tmp_s3 < tmp_s4) ) {
					char *tmp_s5 = NULL;
					char *tmp_s6 = NULL;

					if( (tmp_s5 = strchr(tmp_s3, ')')) 
								== NULL) {
						URL_free( ret );
						return(NULL);
					}
					tmp_s3++;
					tmp_s6 = strchr(tmp_s3, '=');
					if( tmp_s6 > tmp_s5 ) {
						URL_free( ret );
						return(NULL);
					}
					tmp_s6 = (char *) malloc ( tmp_s5 - tmp_s3+1);
					memset( tmp_s6, 0, tmp_s5 - tmp_s3 + 1);
					memcpy( tmp_s6, tmp_s3, tmp_s5 - tmp_s3);
					PKI_STACK_push( ret->attrs, tmp_s6 );

					tmp_s2 = tmp_s3;
				}
				tmp_s = tmp_s4;
				*/
				len = tmp_attrs - tmp_s2;
				ret->path = (char *) malloc ( len );
				memset( ret->path, 0, len);
				memcpy( ret->path, tmp_s2, len - 1);

				if( !ret->addr ) {
					len = (int) (tmp_s2 - tmp_s - 1);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}
			} else {
				if( !ret->addr ) {
					len = (int) 
						((long) tmp_s2 - (long) tmp_s);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}

				tmp_s = tmp_s2;
				ret->path = strdup( tmp_s );
			}
		} else {
			if( !ret->addr )
				ret->addr = strdup( tmp_s );
			ret->path = strdup("/");
		}
	} else if (strncmp( "file://", url_s, 6 ) == 0) {

		ret->port = -1;
		ret->proto = URI_PROTO_FILE;
		len = strlen(&url_s[7]);

		if( len > 0 ) {
			ret->addr = (char *) malloc ( len + 1 );
			memset( ret->addr, 0, len + 1 );
			strncpy( ret->addr, &url_s[7], len );
		} else {
			ret->addr = "";
		}

	} else if (strncmp( "id://", url_s, 5 ) == 0) {

		ret->port = -1;
		ret->proto = URI_PROTO_ID;
		len = strlen(&url_s[5]);

		if( len > 0 ) {
			ret->addr = (char *) malloc ( len + 1 );
			memset( ret->addr, 0, len + 1 );
			strncpy( ret->addr, &url_s[5], len );
		} else {
			ret->addr = "";
		}

		/*
		tmp_s = &url_s[5];

		if((tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s4 = strchr(tmp_s, '/');
			if( tmp_s4 && tmp_s3 < tmp_s4 ) {
				tmp_s2 = strchr( tmp_s, ':' );

				if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
					return (NULL);
				}
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->usr = (char *) malloc (len+1);
				memset( ret->usr, 0, len+1);
				strncpy( ret->usr, tmp_s, len);
				ret->usr[len] = '\x0';
			
				tmp_s = tmp_s2+1;
				tmp_s2 = strchr( tmp_s,'@');

				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->pwd = (char *) malloc (len+1);
				memset( ret->pwd, 0, len+1);
				strncpy( ret->pwd, tmp_s, len);
				ret->pwd[len] = '\x0';

				tmp_s = tmp_s2+1;
			}
		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset( ret->addr, 0, len+1 );

			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
			if( ret->port == 0 ) {
				Error in parsing the port number!
				URL_free ( ret );
				return(NULL);
			}

			if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
				tmp_s =  tmp_s2;
			} else {
				tmp_s = &url_s[7];
			}
		}

		if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
			char *tmp_attrs = NULL;
			tmp_s2++;

			if((tmp_attrs = strchr( tmp_s, '?' )) != NULL) {
				len = strlen( tmp_attrs );
				ret->attrs = (char *) malloc ( len );
				memset( ret->attrs, 0, len );
				memcpy( ret->attrs, tmp_attrs + 1, len);

				len = tmp_attrs - tmp_s2;
				ret->path = (char *) malloc ( len );
				memset( ret->path, 0, len);
				memcpy( ret->path, tmp_s2, len - 1);

				if( !ret->addr ) {
					len = (int) (tmp_s2 - tmp_s - 1);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}
			} else {
				if( !ret->addr ) {
					len = (int) 
						((long) tmp_s2 - (long) tmp_s);
					ret->addr = (char *) malloc (len+1);
					memset( ret->addr, 0, len+1);
					strncpy( ret->addr, tmp_s, len);
					ret->addr[len] = '\x0';
				}

				tmp_s = tmp_s2;
				ret->path = strdup( tmp_s );
			}
		} else {
			if( !ret->addr )
				ret->addr = strdup( tmp_s );
			ret->path = "/";
		}
		*/
	} else if( strncmp("http://", url_s, 6 ) == 0) {
		ret->proto = URI_PROTO_HTTP;
		tmp_s = &url_s[7];

		if((tmp_s3 = strchr(tmp_s, '@')) != NULL ) {
			tmp_s4 = strchr(tmp_s, '/');
			if( tmp_s4 && tmp_s3 < tmp_s4 ) {
				tmp_s2 = strchr( tmp_s, ':' );

				if( !tmp_s2 || (tmp_s3 < tmp_s2)) {
					return NULL;
				}
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->usr = (char *) malloc (len+1);
				memset( ret->usr, 0, len+1);

				strncpy( ret->usr, tmp_s, len);
				ret->usr[len] = '\x0';
			
				tmp_s = tmp_s2+1;
				tmp_s2 = strchr( tmp_s,'@');

				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->pwd = (char *) malloc (len+1);
				memset( ret->pwd, 0, len+1);
				strncpy( ret->pwd, tmp_s, len);
				ret->pwd[len] = '\x0';

				tmp_s = tmp_s2+1;

			}
		}

		if( strchr( tmp_s, ':' )) {
			tmp_s2 = strchr(tmp_s,':');

			len = (int) ( (long) tmp_s2 - (long) tmp_s );
			ret->addr = (char *) malloc (len+1);
			memset( ret->addr, 0, len+1 );

			strncpy( ret->addr, tmp_s, len);
			ret->addr[len] = '\x0';

			tmp_s = tmp_s2+1;
			ret->port = atoi( tmp_s );
			if( (tmp_s2 = strchr( tmp_s, '/' )) != NULL ) {
				tmp_s =  tmp_s2;
			} else {
				tmp_s = &url_s[7];
			}
		} else {
			ret->port = DEFAULT_HTTP_PORT;
		}

		if( strchr( tmp_s, '/' )) {

			tmp_s2 = strchr(tmp_s,'/');

			if( !ret->addr ) {
				len = (int) ( (long) tmp_s2 - (long) tmp_s );
				ret->addr = (char *) malloc (len+1);
				memset( ret->addr, 0, len+1);
				strncpy( ret->addr, tmp_s, len);
				ret->addr[len] = '\x0';
			}

			tmp_s = tmp_s2;
			ret->path = strdup( tmp_s );
		} else {
			if( !ret->addr )
				ret->addr = strdup( tmp_s );
			ret->path = strdup("/");
		}
	} else if( strncmp("pkcs11://", url_s, 8 ) == 0) {
		ret->proto = URI_PROTO_PKCS11;
		ret->port = DEFAULT_PKCS11_PORT;

		tmp_s = &url_s[9];

		if(( tmp_s3 = strstr(tmp_s, "/(" )) != NULL ) {

			len = tmp_s3 - tmp_s;
			ret->addr = PKI_Malloc( len + 1);
			strncpy( ret->addr, tmp_s, len );
			tmp_s = tmp_s3 + 1;

			if(( tmp_s3 = strchr( tmp_s, '?' )) != NULL ) {
				len = tmp_s3 - tmp_s;
				ret->path = PKI_Malloc( len + 1);
				strncpy( ret->path, tmp_s, len );

				tmp_s = tmp_s3 + 1;
				len = strlen( tmp_s );
				ret->attrs = PKI_Malloc ( len + 1);
				strncpy( ret->attrs, tmp_s, len );
			} else {
				len = strlen( tmp_s );
				ret->path = PKI_Malloc( len + 1 );
				strncpy( ret->path, tmp_s, len );
				ret->attrs = strdup( "data" );
			}

		} else if(( tmp_s3 = strchr( tmp_s, '?')) != NULL ) {

			len = tmp_s3 - tmp_s;
			ret->addr = PKI_Malloc( len + 1 );
			strncpy( ret->addr, tmp_s, len );
			tmp_s = tmp_s3 + 1;

			ret->attrs = strdup( tmp_s );
		} else {
			ret->addr = strdup( tmp_s );
			ret->attrs = strdup( "data" );
			ret->path = strdup( "" );
		}

		if ((ret->addr) && (ret->addr[strlen(ret->addr)-1] == '/') ) {
			ret->addr[strlen(ret->addr)-1] = '\x0';
		}

	} else {

		/* No protocol specified, we assume file:// */
		ret->port = -1;
		ret->proto = URI_PROTO_FILE;
		ret->addr = (char *) malloc ( BUFF_MAX_SIZE );
		memset( ret->addr, 0, BUFF_MAX_SIZE );
		strncpy( ret->addr, url_s, BUFF_MAX_SIZE);
	}

	return ret;
}

int URL_free ( URL *url ) {

	if( !url ) return (0);

	if( url->addr ) free (url->addr);
	if( url->usr ) free (url->usr );
	if( url->pwd ) free (url->pwd );
	if( url->path ) free (url->path );
	if( url->url_s ) free (url->url_s);
	if( url->attrs ) free( url->attrs );

	if( url ) free (url);

	return(1);
}
