/* OID management for libpki */

#include <libpki/pki.h>

/*!
 * \brief Create a new OID object
 *
 * Create a new OID by using its name. It returns a PKI_OID
 * pointer if successful, otherwise it returns NULL
 */

PKI_OID *PKI_OID_new( char *oid, char *name, char *descr ) {
	PKI_OID *ret = NULL;
	int nid = NID_undef;

	/* Check if the object already exists */
	if( ((nid = OBJ_sn2nid(name)) != NID_undef) ||
		((nid = OBJ_ln2nid(name)) != NID_undef) )
			ret = OBJ_nid2obj(nid);

	if(!ret) {
		/* If it does not exists, then create it */
		(void) OBJ_create( oid, name, descr );

		if( ((nid = OBJ_sn2nid(name)) != NID_undef) ||
			((nid = OBJ_ln2nid(name)) != NID_undef) )
				ret = OBJ_nid2obj(nid);
	}

	/* If successful it returns the new Object, otherwise it
	   return NULL */
	return ( ret );
}

/*!
 * \brief Free memory associated with a PKI_OID structure
 *
 * This function frees the memory associated with the provided
 * pointer to a PKI_OID structure.
 */

void PKI_OID_free ( PKI_OID *oid ) {

	if( !oid ) return;

	ASN1_OBJECT_free ( oid );

	return;
}

void PKI_OID_free_void ( void *buf ) {
	PKI_OID_free( (PKI_OID *) buf);
}

/*!
 * \brief Retrieve a pointer to an OID
 *
 * This function retrieves an OID pointer from the passed name.
 * Check also the configuration options.
 */

PKI_OID *PKI_OID_get( char *name ) {

	PKI_OID *ret = NULL;
	int nid = NID_undef;

	/* Check if the object already exists */
	if( ((nid = OBJ_sn2nid(name)) != NID_undef) ||
		((nid = OBJ_ln2nid(name)) != NID_undef) )
			ret = OBJ_nid2obj(nid);

	return( ret );
}

/*!
 * \brief Returns a duplicate of the passed PKI_OID structure
 */

PKI_OID *PKI_OID_dup( PKI_OID *a ) {
	PKI_OID *ret = NULL;

	if( !a ) return ( NULL );

	ret = OBJ_dup( a );
	return ( ret );
}

