/* STRING management for libpki */

#include <libpki/pki.h>

/*!
 * \brief Create a new PKI_STRING
 */

PKI_STRING * PKI_STRING_new_null ( int type ) {
	return( PKI_STRING_new( type, NULL, 0 ));
}

PKI_STRING * PKI_STRING_new( int type, char * val, size_t size ) {
	PKI_STRING *ret = NULL;

	switch ( type ) {
		case PKI_STRING_IA5:
			ret = ASN1_IA5STRING_new();
			break;
		case PKI_STRING_UTF8:
			ret = ASN1_UTF8STRING_new();
			break;
		case PKI_STRING_BMP:
			ret = ASN1_BMPSTRING_new();
			break;
		case PKI_STRING_BIT:
			ret = ASN1_BIT_STRING_new();
			break;
		default:
			PKI_log_debug( "PKI STRING type %d not implemented",
				type );
			return ( NULL );
	}

	if( val ) {
		if( size <= 0 ) {
			ASN1_STRING_free ( ret );
			return ( NULL );
		}

		PKI_STRING_set( ret, val, size );
	}

	return ( ret );
}

int PKI_STRING_set( PKI_STRING *s, char *content, size_t size ) {

	int type = 0;

	if( !s || !content ) return ( PKI_ERR );

	if(ASN1_STRING_set( s, content, size ) < 0 ) {
		return( PKI_ERR );
	};

	return ( PKI_OK );
}

int PKI_STRING_get_type( PKI_STRING *s ) {

	int type = PKI_STRING_UNKNOWN;
	int ret = PKI_STRING_UNKNOWN;

	if( !s ) return ( ret );

	type = ASN1_STRING_type ( s );

	switch( type ) {
		case PKI_STRING_IA5:
		case PKI_STRING_UTF8:
		case PKI_STRING_BMP:
		case PKI_STRING_T61:
			ret = type;
			break;
		default:
			ret = PKI_STRING_UNKNOWN;
	}

	return ( ret );
}

char * PKI_STRING_get_utf8( PKI_STRING *s ) {

	char *ret = NULL;

	if( !s ) return ( NULL );

	if((ASN1_STRING_to_UTF8( (unsigned char **) &ret, s )) < 0 ) {
		return( NULL );
	}

	return( ret );
}

void PKI_STRING_free( PKI_STRING *s ) {

	if( !s ) return;

	ASN1_STRING_free ( s );

	return;
}


int PKI_STRING_print( PKI_STRING *s ) {
	return ( PKI_STRING_print_fp( stdout, s ));
}

int PKI_STRING_print_fp( FILE *fp, PKI_STRING *s ) {

	char *str = NULL;

	if( !s ) return ( PKI_ERR );

	if((str = PKI_STRING_get_utf8( s )) == NULL ) {
		return ( PKI_ERR );
	}

	fprintf( fp, "%s", str );

	PKI_Free( str );

	return ( PKI_OK );
}

