/* PKI_TIME */

#include <libpki/pki.h>

PKI_TIME *PKI_TIME_new( long offset ) {
	PKI_TIME *time = NULL;

	if((time = (PKI_TIME *) ASN1_GENERALIZEDTIME_new()) == NULL ) {
		return ( NULL );
	}

	/* Set the time offset - if offset is 0 then it gets the current
	   time */
	X509_gmtime_adj((ASN1_GENERALIZEDTIME *) time, offset);

	return( time );
}

void PKI_TIME_free_void( void *time ) {
	int ret;

	ret = PKI_TIME_free( (PKI_TIME *) time);
	return;
}

int PKI_TIME_free( PKI_TIME *time ) {

	if( !time ) return (PKI_ERR);

	ASN1_TIME_free( (ASN1_TIME *) time );

	return (PKI_OK);
}

char *PKI_TIME_get_parsed ( PKI_TIME *t ) {

	BUF_MEM *bm = NULL;
	BIO *mem = NULL;
	int tmp_ret = 0;
	char *ret = NULL;

	if( !t ) return (NULL);

	if((mem = BIO_new(BIO_s_mem())) == NULL ) {
		return(NULL);
	};

	tmp_ret = ASN1_TIME_print( mem, (ASN1_TIME *) t);
	BIO_get_mem_ptr(mem, &bm);

	if(( ret = PKI_Malloc( bm->length + 1 )) != NULL ) {
		memcpy((char *)ret, bm->data, bm->length);
		ret[bm->length] = '\x0';
	}

	BIO_free( mem );
	return( ret );

}

int PKI_TIME_print ( PKI_TIME *time ) {
	return ( PKI_TIME_print_fp( stdout, time ));
}

int PKI_TIME_print_fp ( FILE *fp, PKI_TIME *time ) {

	BIO *out = NULL;
	if( !time || !fp ) return ( PKI_ERR );

	if((out = BIO_new_fp( fp, BIO_NOCLOSE )) == NULL ) {
		return( PKI_ERR );
	}
	
	ASN1_GENERALIZEDTIME_print(out, time);

	if( out ) BIO_free ( out );

	return (PKI_OK);
}
