/* Extension value building - driver specific part */

#include <libpki/pki.h>

PKI_X509_EXTENSION *PKI_X509_EXTENSION_new( void ) {
	PKI_X509_EXTENSION *ret = NULL;

	ret = X509_EXTENSION_new();
	if( !ret ) return (NULL);

	return (ret);
}

void PKI_X509_EXTENSION_free_void ( void *ext ) {
	PKI_X509_EXTENSION_free ( (PKI_X509_EXTENSION *) ext );
}

void PKI_X509_EXTENSION_free ( PKI_X509_EXTENSION *ext ) {

	if( ext ) X509_EXTENSION_free (ext);

	return;
}


PKI_X509_EXTENSION *PKI_X509_EXTENSION_value_new_profile ( 
		PKI_X509_PROFILE *profile, PKI_CONFIG *oids, 
			PKI_CONFIG_ELEMENT *extNode ) {

	PKI_CONFIG_ELEMENT *valNode = NULL;
	PKI_X509_EXTENSION *ret = NULL;

	xmlChar *type_s = NULL;
	xmlChar *tag_s = NULL;
	xmlChar *oid_s = NULL;
	xmlChar *value_s = NULL;
	xmlChar *name_s = NULL;
	xmlChar *crit_s = NULL;

	PKI_OID *oid = NULL;

	X509V3_CTX v3_ctx;

	char *valString = NULL;
	int crit = 0;
	int nid = NID_undef;

	if( !profile || !extNode ) {
		PKI_log_debug("ERROR, no profile or extNode provided in "
				"PKI_X509_EXTENSION_value_new_profile()");
		return (NULL);
	}

	if((crit_s = xmlGetProp( extNode, BAD_CAST "critical" )) != NULL ) {
		if( strncmp_nocase( (char *) crit_s, "n", 1 ) == 0) {
			crit = 0;
		} else {
			crit = 1;
		}
	}

	if((name_s = xmlGetProp( extNode, BAD_CAST "name" )) == NULL ) {
		PKI_log_debug("ERROR, no name property in node %s", 
							extNode->name);
		if( crit_s ) xmlFree ( crit_s );
		return (NULL);
	}

	if((nid = OBJ_sn2nid( (char *) name_s )) == NID_undef ) {
		PKI_OID *oid = NULL;

		oid = PKI_CONFIG_OID_search ( oids, (char *) name_s );
		if( !oid ) {
			PKI_log_debug( "ERROR, can not create object (%s)!",
				name_s );
			return( NULL );
		}
	}

	if((valString = ( char * ) PKI_Malloc ( BUFF_MAX_SIZE )) == NULL ) {
		PKI_log_debug( "ERROR, Memory Allocation (%s:%d)",
				 __FILE__, __LINE__ );
		if( name_s ) xmlFree ( name_s );
		if( crit_s ) xmlFree (crit_s);
		return( NULL );
	}

	memset( valString, 0, BUFF_MAX_SIZE );
	if( crit == 1 ) {
		snprintf( valString, BUFF_MAX_SIZE -1, "%s", "critical" );
	}

    	for (valNode = extNode->children; valNode; valNode = valNode->next) {
		if ((valNode->type == XML_ELEMENT_NODE) &&
			((strncmp_nocase( (char *) (valNode->name), 
							"value", 5)) == 0)) {

			char tmp[BUFF_MAX_SIZE];

			type_s = xmlGetProp( valNode, BAD_CAST "type" );

			tag_s = xmlGetProp( valNode, BAD_CAST "tag" );
			oid_s = xmlGetProp( valNode, BAD_CAST "oid" );

			value_s = xmlNodeListGetString( profile, 
						valNode->xmlChildrenNode, 0);

			if( oid_s ) {
				/* Let's be sure the OID is created */
				if((oid = PKI_CONFIG_OID_search(oids, 
						(char *) value_s)) == NULL ) {
					PKI_log_debug ("WARNING, no oid "
						"created for %s!", oid_s );
				} else {
					PKI_OID_free ( oid );
				}
			}

			memset( tmp, 0, BUFF_MAX_SIZE );

			if( tag_s ) {
				snprintf( tmp, BUFF_MAX_SIZE - 1, "%s;", tag_s);
			}

			if( type_s == NULL ) {
				if( !oid_s ) {
					strncat( tmp, (char *) value_s, 
						BUFF_MAX_SIZE - strlen( tmp ));
				} else {
					if( value_s && (strlen((char *) value_s) > 0) ) {
						strncat( tmp, (char *) oid_s, 
						 	BUFF_MAX_SIZE - strlen( tmp ) );
						strncat( tmp, ":", 
							BUFF_MAX_SIZE - strlen ( tmp ));
						strncat( tmp, (char *) value_s,
							BUFF_MAX_SIZE - strlen (tmp ));
					} else {
						strncat( tmp, "OID:", 
							BUFF_MAX_SIZE - strlen(tmp));
						strncat( tmp, (char *) oid_s, 
							BUFF_MAX_SIZE - strlen( tmp ));
					}
				}
			} else {
				strncat( tmp, (char *) type_s, 
						BUFF_MAX_SIZE - strlen( tmp ));
				if( value_s && (strlen((char*)value_s) > 0) ) {
					strncat( tmp, ":", 
						BUFF_MAX_SIZE - strlen( tmp ));
					strncat( tmp, (char *) value_s, 
						BUFF_MAX_SIZE - strlen( tmp ));
				}
			}
				

			/*
			if( type && value ) {
				 snprintf( tmp, BUFF_MAX_SIZE - 1, "%s:%s", type, value );
			} else if( value ) {
				 snprintf( tmp, BUFF_MAX_SIZE - 1, "%s", value );
			}
			*/

			if( tmp && strlen( valString ) > 0 ) {
				strncat( valString, ",", BUFF_MAX_SIZE - 1);
			}
			strncat( valString, (char *) tmp, BUFF_MAX_SIZE - 1 );

			if( type_s ) xmlFree ( type_s );
			if( oid_s ) xmlFree ( oid_s );
			if( tag_s ) xmlFree ( tag_s );
			if( value_s ) xmlFree ( value_s );
        	}
	}
	PKI_log_debug("INFO, Encoding %s=%s", name_s, valString);

	// memset( &v3_ctx, 0, sizeof( v3_ctx ));
	// ret = X509V3_EXT_conf(NULL, &v3_ctx, (char *) name_s, 
	// 					(char *) valString);
	v3_ctx.db = NULL;
	v3_ctx.db_meth = NULL;
	v3_ctx.issuer_cert = NULL;
	v3_ctx.subject_cert = NULL;
	v3_ctx.subject_req = NULL;
	v3_ctx.crl = NULL;

	memset( &v3_ctx, 0, sizeof( v3_ctx ));
	ret = X509V3_EXT_conf(NULL, &v3_ctx, (char *) name_s, 
						(char *) valString);
	// ret = X509V3_EXT_conf(NULL, NULL, (char *) name_s, 
	//					(char *) valString);
	if( !ret ) {
		ERR_print_errors_fp ( stdout );
	}

	if( name_s ) xmlFree ( name_s );
	if( crit_s ) xmlFree (crit_s );

	if( valString ) PKI_Free ( valString );

	return ( ret );
}
