/* openssl/pki_x509_name.c */

#include <libpki/pki.h>

PKI_X509_NAME *PKI_X509_NAME_new_null ( void ) {

	PKI_X509_NAME *ret = NULL;

	ret = (PKI_X509_NAME *) X509_NAME_new();
	return (ret);
}

int PKI_X509_NAME_free( PKI_X509_NAME *name ) {

	if( name == NULL ) return(1);

	X509_NAME_free ( (X509_NAME *) name );

	return(1);
}

PKI_X509_NAME *PKI_X509_NAME_new ( char *name ) {

	PKI_X509_NAME *ret = NULL;
	int status = 0;

	char *token = NULL;
	char *start = NULL;
	char *pnt = NULL;

	unsigned long ctype = MBSTRING_ASC;

	if((ret = PKI_X509_NAME_new_null()) == NULL ) {
		PKI_log_debug("ERROR, can not create a new X509_NAME!");
		return (NULL);
	}

	start = name;
	pnt = start;
	while( ( pnt ) && (pnt < start + strlen(start)) && (status != 5) ) {
		if ( status == 0 ) {
			if( *pnt == ' ' ) {
				pnt++;
				continue;
			} else if ( (*pnt == ',') || (*pnt == '/') ||
						(*pnt == ';')) {
				break;
			} else {
				start = pnt;
				pnt++;
				status = 1;
				continue;
			}
		} else if ( status == 1 ) {
			if( *pnt == '\\' ) {
				status = 2;
				pnt++;
				continue;
			} else if ( *pnt == '=' ) {
				pnt++;
				status = 3;
				continue;
			} else if ( (*pnt == ',') || (*pnt == '/') ||
						(*pnt == ';')) {
				break;
			} else {
				pnt++;
				continue;
			}
		} else if ( status == 2 ) {
			pnt++;
			status = 1;
			continue;
		} else if ( status == 3 ) {
			if( *pnt == '\\' ) {
				status = 4;
				pnt++;
				continue;
			} else if ( (*pnt == ',') || (*pnt == '/') ||
					(*pnt == ';') || *(pnt+1) == 0) {

				int len, i, k;

				char *key = NULL;
				char *val = NULL;

				len = pnt-start;
				if( *(pnt+1) == 0 ) len++;

				token = (char *) malloc(len+1);
				memset(token, 0, len);
				strncpy(token, start, len);
				token[len] = '\x0';
				i=0;
				for(k=0;k<len;k++) {
					if( *(token+k) != '\\' ) {
						*(token+i) = *(token+k);
					} else {
						*(token+i) = *(token+k+1);
						k++;
					}
					i++;
				}
				token[i] = 0;
				key = malloc ( len );
				val = malloc ( len );
				sscanf(token, "%[^=]=%[^\\]", key, val);

				if (!X509_NAME_add_entry_by_txt(
					(X509_NAME *) ret, key, 
					ctype, (const unsigned char *) val, -1, -1, 0)) {
                                		if(name) X509_NAME_free (
							(X509_NAME *) name);
						free(token);
						free(key);
						free(val);
				}

				free( key );
				free( val );
				free ( token );

				token = NULL;

				pnt++;
				if( *pnt != 0 ) status = 0;
				continue;
			} else {
				pnt++;
				continue;
			}
		} else if ( status == 4 ) {
			pnt++;
			status = 3;
			continue;
		}
	}

	if( status != 3 ) {
		if( ret ) PKI_X509_NAME_free (ret);
		return (NULL);
	}

	return(ret);
}

PKI_X509_NAME *PKI_X509_NAME_dup ( PKI_X509_NAME *name ) {

	if( !name ) return (NULL);

	return X509_NAME_dup ( name );
}

char *PKI_X509_NAME_get_parsed ( PKI_X509_NAME *name ) {

	char buf[BUFF_MAX_SIZE];
	char *ret = NULL;

	if(!name) return (NULL);

	memset(buf, 0, sizeof( buf ));
	X509_NAME_oneline(name, buf, sizeof buf);
	if( strlen( buf ) > 0 ) {
		ret = PKI_Malloc( strlen(buf)+1);
		strncpy(ret, buf, strlen(buf));
	}

	return( ret );
}

