/* src/pki_cred.c */

#include <libpki/pki.h>

/*!
 * \brief Allocates a new PKI_CRED structure
 *
 * Allocates memory for a new PKI_CRED structure. The returned data
 * structure is already zeroized.
 *
 * The function returns a pointer to a PKI_CRED structure in case
 * of success, otherwise it returns NULL.
 */

PKI_CRED *PKI_CRED_new ( void ) {

	PKI_CRED *c = NULL;

	c = (PKI_CRED *) PKI_Malloc (sizeof(PKI_CRED));

	return (c);
}

/*!
 * \brief Free a PKI_CRED memory region
 *
 * This function frees a PKI_CRED data structure. The internal data
 * is also freed (no need to free the internal structures before
 * calling this function.
 *
 * No value is returned (void).
 */

void PKI_CRED_free( PKI_CRED *cred ) {

	if( !cred ) return;

	if( cred->password != NULL ) PKI_Free ( (void *) cred->password );
	if( cred->username != NULL ) PKI_Free ( (void *) cred->username );
	if( cred->prompt_info != NULL ) PKI_Free ( (void *) cred->prompt_info );

	PKI_Free( cred );

	return;
}

