/* OpenCA libpki package
* (c) 2000-2007 by Massimiliano Pala and OpenCA Group
* All Rights Reserved
*
* ===================================================================
* Released under OpenCA LICENSE
*/

#include <libpki/pki.h>

PKI_MEM *PKI_MEM_new_null ( void ) {

	PKI_MEM *ret = NULL;

	ret = PKI_Malloc ( sizeof( PKI_MEM ));
	if( !ret ) return (NULL);

	return (ret);
}

PKI_MEM *PKI_MEM_new ( size_t size ) {
	PKI_MEM *ret = NULL;

	ret = PKI_MEM_new_null();
	if( !ret ) return (NULL);

	ret->data = PKI_Malloc ( size );
	if( !ret->data ) {
		PKI_MEM_free ( ret );
		return (NULL);
	}
	ret->size = size;

	return(ret);
}

void PKI_MEM_free_void ( void *buf ) {
	PKI_MEM_free( (PKI_MEM *) buf);
}

int PKI_MEM_free ( PKI_MEM *buf ) {

	if( !buf ) return (0);
	if( buf->data ) PKI_Free ( buf->data );
	
	PKI_Free ( buf );

	return 1;
}

int PKI_MEM_grow( PKI_MEM *buf, size_t data_size ) {

	size_t new_size = 0;

	if( !buf ) return (PKI_ERR);

	if( buf->data == NULL ) {
		buf->data = PKI_Malloc ( data_size );
		if( !buf->data ) return (PKI_ERR);
		
		buf->size = data_size;
	} else {
		new_size = buf->size + data_size;
		buf->data = realloc ( buf->data, new_size );
		buf->size = new_size;
	}

	return (buf->size);
}

int PKI_MEM_add( PKI_MEM *buf, char *data, size_t data_size ) {
	
	size_t curr_size = 0;

	if( (!buf) || (!data) || (data_size == 0) )
		return (PKI_ERR);

	curr_size = PKI_MEM_get_size ( buf );

	if( PKI_MEM_grow( buf, data_size ) == 0 ) return (PKI_ERR);
	memcpy(buf->data + curr_size, data, data_size );

	return( PKI_OK );
}

char * PKI_MEM_get_data( PKI_MEM *buf ) {

	if (!buf ) return (PKI_ERR);

	return( buf->data );
}

size_t PKI_MEM_get_size( PKI_MEM *buf ) {
	if( !buf || !buf->data ) return (0);

	return( buf->size );
}

void *PKI_Malloc( size_t size ) {

	void *ret = NULL;

	if ( size <= 0 ) return NULL;

	if((ret = (void *) malloc( size )) != NULL ) {
		memset(ret, 0, size );
	};

	return (ret);
}

void PKI_Free( void *ret ) {
	
	if( ret == NULL ) return;

	free ( ret );

	return;
}

