/* Profile management for libpki */

#include <libpki/pki.h>

char * PKI_X509_PROFILE_get_value ( PKI_X509_PROFILE *doc, char *path ) {
	return PKI_CONFIG_get_value ( (PKI_CONFIG *) doc, path );
}

char *PKI_X509_PROFILE_get_name ( PKI_X509_PROFILE *doc ) {
	char search[BUFF_MAX_SIZE];

	snprintf(search, (size_t) BUFF_MAX_SIZE, "/%s:profile/%s:name", 
			PKI_NAMESPACE_PREFIX, PKI_NAMESPACE_PREFIX );

	return PKI_X509_PROFILE_get_value( doc, search );
}

PKI_X509_PROFILE * PKI_X509_PROFILE_load(char *urlPath) {

    PKI_X509_PROFILE *doc = NULL;
    // xmlNode *root_element = NULL;
    URL *url = NULL;

    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION

    if( urlPath ) {
        url = URL_new( urlPath );
    } else {
        url = URL_new ( PKI_DEFAULT_PROFILE_DIR );
    }

    if( !url ) {
	PKI_log_debug("ERROR, can not parse URL when loading profile (%s)!\n",
							urlPath );
	return(PKI_ERR);
    }

    /*parse the file and get the DOM */
    doc = (PKI_X509_PROFILE *) xmlReadFile(url->addr, NULL, 0);

    if (doc == NULL) {
        PKI_log_debug("ERROR, could not parse file %s\n", url->addr);
	return (PKI_ERR);
    }

    return( doc );
}

void PKI_X509_PROFILE_free_void ( void * doc ) {
	PKI_X509_PROFILE_free( (PKI_X509_PROFILE *) doc );
}

int PKI_X509_PROFILE_free ( PKI_X509_PROFILE * doc ) {
	if( !doc ) return (PKI_OK);

	xmlFreeDoc( doc );

	/*
	*Free the global variables that may
	*have been allocated by the parser.
	*/
	//xmlCleanupParser();

	return( PKI_OK );
}


