/*
 * OCSP responder
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 2001
 *
 * Copyright (c) 2001 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include <libpki/pki.h>

/* Functions */
BIO *http_connect( URL *url ) {

	BIO *cbio = NULL;
	char buf[1024];
	int sock = -1;

	if( !url || !(url->addr) ) {
		PKI_log( PKI_LOG_ERR, "Missing address for HTTP connect!");
		return ( NULL );
	}

	if((sock = inet_connect( url )) < 1 ) {
		PKI_log( PKI_LOG_ERR, "ERROR, can not connect to server %s",
						url->addr );
		return ( NULL );
	}

	/*
	cbio = BIO_new(BIO_s_socket);
	BIO_set_fd( cbio, sock, 0);
	*/

	if((cbio = BIO_new_fd( sock, 0 )) == NULL ) {
		PKI_log( PKI_LOG_ERR, "ERROR, can not assign FD %d to BIO",
							sock );
		inet_close ( sock );
		return( NULL );
	}

	/* We build the "host:port" string to be used */
	/*
	memset(buf, '\x0', sizeof(buf));
        snprintf( buf, sizeof(buf), "%s:%d", url->addr, url->port );

	if((cbio = BIO_new_connect(buf)) == NULL ) {
		PKI_log( PKI_LOG_ERR, "Can not connect to %s", url->addr);
		return ( NULL );
	}

	if (BIO_do_connect(cbio) <= 0) {
		PKI_log( PKI_LOG_ERR, "Can not connect to %s", url->addr);
		BIO_free_all(cbio);
		return NULL;
	}
	*/

	return cbio;
}

BUF_MEM *http_get_data ( BIO *in, ssize_t max_size ) {

	BUF_MEM *buf = NULL;
	unsigned long fullsize = 0;
	unsigned long newsize  = 0;

	if( !in )
		return NULL;

	/* get subject name from bio using recommended OpenSSL template */
	buf = BUF_MEM_new();
	for (;;) {
		if ((buf == NULL) || (!BUF_MEM_grow(buf, fullsize+512))) {
			PKI_log( PKI_LOG_ERR, "Memory Allocation Err (%s:%d)",
					__FILE__, __LINE__ );
			return ( NULL );
		}

		newsize   = BIO_read(in, &(buf->data[fullsize]), 512);
		fullsize += newsize;
		if (newsize == 0) break;

		if (newsize < 0) {
			/* Error Reading from buf */
			BUF_MEM_free( buf );
			return NULL;
		}

		if( (max_size > 0) && (fullsize > max_size)) {
			/* Max Reading size exceeded */
			// fprintf( stderr, 
			// 	"ERROR::HTTP::Read::Max read size exceeded "
			// 	" [ %d ]", max_size );
			BUF_MEM_free( buf );
			return NULL;
		}
	}

	/* Make sure the buf ends with a \x0 value */
	buf->data[fullsize] = '\x0';

	return buf;
}

PRQP_RESP *PRQP_http_get_resp ( URL *url, PRQP_REQ *req,
					unsigned long max_size ) {
	BIO *sock = NULL;
	BIO *mem = NULL;
	BUF_MEM *buf = NULL;
	PRQP_RESP *resp = NULL;
	int len = 0;

	static char req_txt[] = 
			"POST %s HTTP/1.1\r\n"
			"Host: %s\r\n"
			"Content-Type: application/prqp-request\r\n"
			"Content-Length: %d\r\n\r\n";

	len = i2d_PRQP_REQ(req, NULL );

	sock = http_connect( url );
	if( !sock ) return NULL;

	/* Print the "GET" command to the server and wait for the
	   answer */
	if( url->path ) {
		BIO_printf(sock, req_txt, url->path, url->addr, len);
	} else {
		BIO_printf(sock, req_txt, "/", url->addr, len);
	}

	// fprintf( stderr, req_txt, url->path, len );

	i2d_PRQP_REQ_bio ( sock, req );

	/* Get HTTP data */
	if((buf = http_get_data ( sock, max_size )) == NULL ) {
		fprintf( stderr, "ERROR -- can not read http data.\n");
	}

	/*
	fprintf( stderr, "---BEGIN RECEIVED DATA---\n" );
	if( buf && buf->data ) {
		fprintf( stderr, "%s\n---END OF RECEIVED DATA---\n", buf->data);
	}
	*/

	/* Release the socket */
	if( sock ) BIO_free_all (sock);
	
	if( (mem=BIO_new(BIO_s_mem())) == NULL ) {
		if( buf ) BUF_MEM_free ( buf );
		return( NULL );
	}

	BIO_set_mem_buf(mem,buf,BIO_CLOSE);

	if(( parse_http_headers( mem )) == 0 ) {
		if( buf ) BUF_MEM_free ( buf );
		return (NULL);
	}

	/*
	fprintf( stderr, "BUFF LEN=>%d - PENDING => %d\n", 
			buf->length, BIO_pending( mem ) );
	fprintf( stderr, "BUFF=>%s\n", buf->data );
	*/

	// if((resp=(PRQP_RESP *) PEM_read_bio_PRQP_RESP(mem)) == NULL) {
	// 	fprintf(stderr, "ERROR, can not load PRQP_RESPONSE!\n");

	if((resp = d2i_PRQP_RESP_bio( mem, NULL )) == NULL ) {
		if( buf ) BUF_MEM_free ( buf );
		return (NULL);
	}

	return ( resp );
}


BUF_MEM *http_get ( URL *url, unsigned long max_size, char *version ) {
	BIO *in = NULL;
	BUF_MEM *buf = NULL;
	char *def_version = "1.1";

	char *get_headers[] = {
		"GET %s HTTP/%s\r\n",
		"Host: %s\r\n",
		NULL
	};

	in = http_connect( url );
	if( !in ) return NULL;

	if( version == NULL ) {
		version = def_version;
	}

	/* Print the "GET" command to the server and wait for the
	   answer */
	BIO_printf(in, get_headers[0], url->path, version);
	// fprintf( stderr, get_headers[0], url->path, version );

	/* For HTTP/1.1 let's specify the server name as well */
	if( strncmp_nocase("1.1", version, 3) == 0 ) {
		// fprintf( stderr, get_headers[1], url->addr );
		BIO_printf( in, get_headers[1], url->addr );
	}

	/* Print EMPTY line for the end of the HEADERS */
	BIO_printf(in, "\r\n");
	// fprintf( stderr, "\r\n");

	/* Get HTTP data */
	if((buf = http_get_data ( in, max_size )) == NULL ) {
		// fprintf( stderr, "ERROR -- can not read http data.\n");
	}

	/*
	if( buf && buf->data ) {
		fprintf( stderr, "---BEGIN RECEIVED DATA---\n" );
		fprintf( stderr, "%s\n---END OF RECEIVED DATA---\n", buf->data);
	}
	*/

	/* Release the socket */
	BIO_free_all (in);
	
	return buf;
}

int parse_http_headers ( BIO *in ) {

	int head_lines = 0;
	char linebuf[1024];

	head_lines = 0;
	do {
		int read_code = 0;

		memset(linebuf, '\x0', sizeof linebuf);
        	if((read_code = BIO_gets(in, linebuf, sizeof(linebuf))) < 0) {
			// fprintf( stderr, "HTTP - Error (%d) retrieving data"
			// 		" (head_lines=%d)", 
			// 			read_code, head_lines );
			return 0;
        	}

		/* Let's check if the first line has good code */
		if ( ( head_lines == 0 ) && 
			( strncmp_nocase( &(linebuf[9]), "200", 3) != 0 )) {
				/* Got an error - probably not found (?) */
				// fprintf( stderr, "HTTP - Error retrieving data"
				// 	" (%s)", linebuf );
				return 0;
		} else {
			head_lines++;
		}
	}
	while( (linebuf[0] != '\r') && (linebuf[0] != '\n') );

	return 1;

}

