/* 
 * PKI Resource Query Protocol Message implementation
 * (c) 2006 by Massimiliano Pala and OpenCA Group
 * All Rights Reserved
 *
 * This software is released under the GPL2 License included
 * in the archive. You can not remove this copyright notice.
 */
                                                                                
#include <libpki/pki.h>

static char *prqp_exts[] = {
	/* PKIX defined */
	"1.3.6.1.5.5.7.48.1", "ocsp", "OCSP Service",
	"1.3.6.1.5.5.7.48.2", "caIssuers", "CA Information",
	"1.3.6.1.5.5.7.48.3", "timeStamping", "TimeStamping Service",
	/* PKIX - not yet defined */
	"1.3.6.1.5.5.7.48.10", "dvcs", "DVCS Service",
	"1.3.6.1.5.5.7.48.11", "scvp", "SCVP Service",
	/* Repositories */
	"1.3.6.1.5.5.7.48.20", "certificateRepository", "Certificate Repository",
	"1.3.6.1.5.5.7.48.21", "crlRepository", "CRL Repository",
	"1.3.6.1.5.5.7.48.22", "crossCertRepository", "Cross Certificate Repository",
	/* Gateways */
	"1.3.6.1.5.5.7.48.40", "cmsGateway", "CMS Gateway",
	"1.3.6.1.5.5.7.48.41", "scepGateway", "SCEP Gateway",
	"1.3.6.1.5.5.7.48.42", "xkmsGateway", "XKMS Gateway",
	/* General Services */
	"1.3.6.1.5.5.7.48.60", "httpRevokeCertificate", "HTTP Certificate Revocation Service",
	"1.3.6.1.5.5.7.48.61", "httpRequestCertificate", "HTTP Certificate Revocation Service",
	"1.3.6.1.5.5.7.48.62", "httpSuspendCertificate", "HTTP Certificate Suspension Service",
	/* Trust Anchors Publishing */
	"1.3.6.1.5.5.7.48.70", "trustAnchors", "Trust Anchors Publishing Point",
	/* Other Services */
	"1.2.826.0.1.3344810.10.2", "webdavCert", "Webdav Certificate Validation Service",
	"1.2.826.0.1.3344810.10.3", "webdavRev", "Webdav Certificate Revocation Service",
	/* Other PKI */
	"2.5.29.27", "deltaCrl", "Delta CRL Base Address",
	"2.5.29.31", "crl", "CRL Repository",
	/* End of the List */
       	NULL, NULL, NULL
};

char *PRQP_STATUS_STRING[] = {
	PRQP_STATUS_STRING_OK,
	PRQP_STATUS_STRING_BAD_REQUEST,
	PRQP_STATUS_STRING_CA_NOT_PRESENT,
	PRQP_STATUS_STRING_SYS_FAILURE
};

char *i2s_ASN1_IA5STRING(X509V3_EXT_METHOD *method, ASN1_IA5STRING *ia5) {
        char *tmp;
        if(!ia5 || !ia5->length) return NULL;
        if(!(tmp = OPENSSL_malloc(ia5->length + 1))) {
                return NULL;
        }
        memcpy(tmp, ia5->data, ia5->length);
        tmp[ia5->length] = 0;
        return tmp;
}

ASN1_IA5STRING *s2i_ASN1_IA5STRING(X509V3_EXT_METHOD *method,
             X509V3_CTX *ctx, char *str) {

        ASN1_IA5STRING *ia5;
        if(!str) {
                return NULL;
        }
        if(!(ia5 = M_ASN1_IA5STRING_new())) goto err;
        if(!ASN1_STRING_set((ASN1_STRING *)ia5, (unsigned char*)str,
                            strlen(str))) {
                M_ASN1_IA5STRING_free(ia5);
                goto err;
        }
// #ifdef CHARSET_EBCDIC
//         ebcdic2ascii(ia5->data, ia5->data, ia5->length);
// #endif /*CHARSET_EBCDIC*/
        return ia5;
        err:
        return NULL;
}

int CERT_IDENTIFIER_cmp ( CERT_IDENTIFIER *a, CERT_IDENTIFIER *b) {

	int ret = 0;

	EXTENDED_CERT_INFO * aInfo = NULL;
	EXTENDED_CERT_INFO * bInfo = NULL;
	
	char abuf[1024];
	char bbuf[1024];

	if( !a || !a->hashAlgorithm || !a->basicCertId ) return (-10);
	if( !b || !b->hashAlgorithm || !b->basicCertId ) return (-20);

	/*
		OBJ_obj2txt(abuf, sizeof(abuf), a->hashAlgorithm->algorithm,0);
		OBJ_obj2txt(bbuf, sizeof(bbuf), b->hashAlgorithm->algorithm,0);
		fprintf(stderr, "%s != %s\n", abuf, bbuf);
	*/
	if((ret = OBJ_cmp(a->hashAlgorithm->algorithm, 
				b->hashAlgorithm->algorithm)) != 0 ) {
		return 1;
	}

	if( !a->basicCertId->issuerNameHash ) return ( -11 );
	if( !b->basicCertId->issuerNameHash ) return ( -21 );
	/*
		fprintf(stderr, "%s != %s\n", i2s_ASN1_OCTET_STRING(NULL, a->issuerNameHash), i2s_ASN1_OCTET_STRING(NULL, b->issuerNameHash));
	*/
	if((ret = ASN1_OCTET_STRING_cmp(a->basicCertId->issuerNameHash, 
				b->basicCertId->issuerNameHash)) != 0 ) {
		return 2;
	};

	if((ret = ASN1_INTEGER_cmp( a->basicCertId->serialNumber,
			b->basicCertId->serialNumber)) != 0 ) {
		return 3;
	};

	if( a->extInfo && b->extInfo ) {
		aInfo = a->extInfo;
		bInfo = b->extInfo;

		if((ret = ASN1_OCTET_STRING_cmp(aInfo->certificateHash, 
				bInfo->certificateHash)) != 0 ) {
			return 4;
		};

		if((ret = ASN1_OCTET_STRING_cmp(aInfo->subjectKeyHash, 
				bInfo->subjectKeyHash)) != 0 ) {
			return 5;
		};

		if( aInfo->subjectKeyId && bInfo->subjectKeyId ) {

			/*
			if((ret = ASN1_OCTET_STRING_cmp(aInfo->subjectKeyId, 
				bInfo->subjectKeyId)) != 0 ) {
				PKI_log_debug("PRQP_CMP (%d): a=%s, b=%s",
					ret,
					i2s_ASN1_OCTET_STRING( NULL, 
						aInfo->subjectKeyId), 
					i2s_ASN1_OCTET_STRING( NULL,
						bInfo->subjectKeyId ));
				return 6;
			};
			*/
		}

		if( aInfo->issuerKeyId && bInfo->issuerKeyId ) {

			if((ret = ASN1_OCTET_STRING_cmp(aInfo->issuerKeyId, 
				bInfo->issuerKeyId)) != 0 ) {
				return 7;
			};
		}
	}
		
	/*
		fprintf(stderr, "%s != %s\n", i2s_ASN1_OCTET_STRING(NULL, a->issuerKeyHash), i2s_ASN1_OCTET_STRING(NULL, b->issuerKeyHash));
	*/

	return 0;
}

int PRQP_init_all_services ( void ) {

	int i, ret;

        i = 0;
        while( prqp_exts[i] && prqp_exts[i+1] ) {
                if((ret = OBJ_create(prqp_exts[i], prqp_exts[i+1], 
				prqp_exts[i+2])) == NID_undef) {
                        return 0;
                }
                i = i+3;
        }

        return 1;

}


PKI_OID * PRQP_s2oid( char *ss ) {

	int i, nid;

	nid = OBJ_sn2nid( ss );
	if( nid != NID_undef ) {
		return ((PKI_OID *) OBJ_nid2obj(nid));
	}

        i = 0;
        while( prqp_exts[i] && prqp_exts[i+1] ) {
		if( strncmp( ss, prqp_exts[i+1], strlen(prqp_exts[i+1])) 
								== 0 ) {
                        nid = OBJ_sn2nid( prqp_exts[i+1] );
			if( nid != NID_undef ) {
				return ((PKI_OID *) OBJ_nid2obj(nid));
			}
		}
		i=i+3;
	}

	return( NULL );
}

const char *PRQP_oid2s ( PKI_OID *a ) {

	int nid, list_nid, i;

	if( !a ) return (0);

	nid = OBJ_obj2nid( a );
	if( nid != NID_undef ) {
		return ( OBJ_nid2sn( nid ) );
	}

        i = 0;
        while( prqp_exts[i] && prqp_exts[i+1] ) {
		list_nid = OBJ_sn2nid( prqp_exts[i+1] );
		if( nid == list_nid ) {
			return( prqp_exts[i+1] );
		}
		i=i+3;
	}
	return( NULL );
}

int PRQP_srv_cmp( PKI_OID *a, PKI_OID *b ) {
	int nid_a, nid_b;

	nid_a = OBJ_obj2nid( a );
	nid_b = OBJ_obj2nid( b );

	if( (nid_a == NID_undef) || (nid_b == NID_undef )) {
		return(-99);
	}

	if( nid_a == nid_b ) {
		return (0);
	}

	return ( 1 );
}

int PRQP_obj2id ( ASN1_OBJECT *a ) {
	char *str_type;
	int ret = PKI_RESOURCE_TYPE_UNKNOWN;

	if( !a ) return( PKI_RESOURCE_TYPE_UNKNOWN );

	return (OBJ_obj2nid (a));

/*
	if((str_type = OBJ_obj2nid (a)) == NULL )
		return ( PKI_RESOURCE_TYPE_UNKNOWN );

	if ( strcmp (str_type, SERVICE_TYPE_OCSP_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_OCSP;
	} else if ( strcmp (str_type, SERVICE_TYPE_CRL_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_CRL;
	} else if ( strcmp (str_type, SERVICE_TYPE_TIMESTAMPING_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_TIMESTAMPING;
	} else if ( strcmp (str_type, SERVICE_TYPE_DVCS_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_DVCS;
	} else if ( strcmp (str_type, SERVICE_TYPE_SCVP_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_SCVP;
	} else if ( strcmp (str_type, SERVICE_TYPE_REVOKE_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_REVOKE;
	} else if ( strcmp (str_type, SERVICE_TYPE_SUBSCRIBE_OID_STRING) == 0 ) {
		ret = PKI_RESOURCE_TYPE_SUBSCRIBE;
	} else {
		ret = PKI_RESOURCE_TYPE_UNKNOWN;
	}

	return( ret );
	*/
}

const char * PRQP_id2s ( int d ) {
	char *str_type;
	int ret = PKI_RESOURCE_TYPE_UNKNOWN;
	int i = 0;
	int nid = 0;

        while( prqp_exts[i] && prqp_exts[i+1] ) {
                nid = OBJ_sn2nid( prqp_exts[i+1]);

		if(nid == d ) return (OBJ_nid2sn( nid ));
		i=i+3;
	}

	return ( OBJ_nid2sn( d ));

/*
	switch( d ) {
		case PKI_RESOURCE_TYPE_OCSP:
			return(SERVICE_TYPE_OCSP_OID_STRING);
			break;
		case PKI_RESOURCE_TYPE_CRL:
			return(SERVICE_TYPE_CRL_OID_STRING);
			break;
		case PKI_RESOURCE_TYPE_TIMESTAMPING:
			return(SERVICE_TYPE_TIMESTAMPING_OID_STRING);
			break;
		case PKI_RESOURCE_TYPE_DVCS:
			return(SERVICE_TYPE_DVCS_OID_STRING);
			break;
		case PKI_RESOURCE_TYPE_SCVP:
			return(SERVICE_TYPE_SCVP_OID_STRING);
			break;
		case PKI_RESOURCE_TYPE_REVOKE:
			return(SERVICE_TYPE_REVOKE_OID_STRING);
			break;
		case PKI_RESOURCE_TYPE_SUBSCRIBE:
			return(SERVICE_TYPE_SUBSCRIBE_OID_STRING);
			break;
		default:
			return(NULL);
	}

	return(NULL);
	*/
}

CERT_IDENTIFIER * PRQP_CERTID_new ( PKI_X509_CERT *caCert, 
		PKI_X509_CERT *issuerCert, PKI_X509_CERT *issuedCert,
		char *subject_s, char *serial_s, PKI_DIGEST_ALG * dgst ) {

        PKI_X509_NAME *s_name = NULL;
        PKI_X509_NAME *i_name = NULL;
        ASN1_INTEGER *serial = NULL;
        ASN1_BIT_STRING *ikey = NULL;
        ASN1_BIT_STRING *caKeyHash = NULL;
        ASN1_BIT_STRING *issKeyHash = NULL;
        ASN1_BIT_STRING *cHash = NULL;
	ASN1_OCTET_STRING *skid = NULL;
	ASN1_OCTET_STRING *ikid = NULL;

	unsigned int i;
	unsigned char md[EVP_MAX_MD_SIZE];

	CERT_IDENTIFIER *ret = NULL;

        if(!dgst) dgst = (PKI_DIGEST_ALG *) PKI_DIGEST_ALG_SHA1;

	/* Now get the IssuerName and the Serial of the Certificate x */
	if( caCert ) {
		X509 *xx = NULL;

		/* Get the subject and issuer Key identifiers */
		/*
		skid = X509_get_ext_d2i((X509 *) caCert, 
				NID_subject_key_identifier, NULL, NULL );
		if(( ikid = X509_get_ext_d2i((X509 *) caCert, 
			NID_authority_key_identifier, NULL, NULL )) == NULL){
		*/
		xx = (X509 *) caCert;
		skid = xx->skid;
		if( xx->akid ) {
			ikid = xx->akid->keyid;
		};

		/* If we have the caCert, let's see if it has the skid that
		   we'll use as an akid */
		/*
			if( issuerCert ) {
				ikid = X509_get_ext_d2i( (X509 *) issuerCert, 
					NID_subject_key_identifier, NULL, NULL);
			}
		*/
		// }

		s_name = (X509_NAME *) 
			PKI_X509_CERT_get_data( caCert, PKI_X509_SUBJECT );

		i_name = (X509_NAME *)
			PKI_X509_CERT_get_data( caCert, PKI_X509_ISSUER );

		serial = (ASN1_INTEGER *) 
			PKI_X509_CERT_get_data( caCert, PKI_X509_SERIAL);

		/* calculate the certificate Hash */
		if((cHash = ASN1_OCTET_STRING_new()) == NULL ) {
			return(NULL);
		} else if (!X509_digest((X509 *)caCert, 
						(EVP_MD *) dgst, md, &i)) {
			ASN1_OCTET_STRING_free( cHash );
			return(NULL);
		}

		if (!(ASN1_OCTET_STRING_set(cHash, md, i))) {
			ASN1_OCTET_STRING_free( cHash );
			return( NULL );
		}

		/* Calculate the Hash of the certificate Key */
		if((caKeyHash = ASN1_OCTET_STRING_new()) == NULL ) {
			if( cHash ) ASN1_OCTET_STRING_free ( cHash );
			return ( NULL );
		}

		if(X509_pubkey_digest((X509 *) caCert, 
					(EVP_MD *)dgst, md, &i) == 0) {
			/* Error in calculating the digest! */
			if( caKeyHash ) ASN1_OCTET_STRING_free( caKeyHash );
			if( cHash ) ASN1_OCTET_STRING_free( cHash );
			return (NULL);
		}

	        if (!(ASN1_OCTET_STRING_set(caKeyHash, md, i))) {
			if( caKeyHash ) ASN1_OCTET_STRING_free( caKeyHash );
			if( cHash ) ASN1_OCTET_STRING_free( cHash );
			return( NULL );
		}

        } else {
		if( serial_s ) {
                	serial = s2i_ASN1_INTEGER( NULL, serial_s);
		}

		if( subject_s ) {
			s_name = PKI_X509_NAME_new( subject_s );
			if(!s_name) {
				PKI_log_debug("%s:%d::Can not parse X509 NAME "
					"%s!", __FILE__, __LINE__, subject_s );
			}
		} else if( issuedCert ) {
			s_name = PKI_X509_CERT_get_data( issuedCert, 
							PKI_X509_ISSUER );
		} else {
			if( serial ) ASN1_INTEGER_free ( serial );
			return (NULL);
		}
	}

	if( issuerCert ) {
		/* Calculate the Hash of the certificate Key */
		if((issKeyHash = ASN1_OCTET_STRING_new()) == NULL ) {
			return ( NULL );
		}

		if(X509_pubkey_digest((X509 *) issuerCert, 
					(EVP_MD *)dgst, md, &i) == 0) {
			/* Error in calculating the digest! */
			if( issKeyHash ) ASN1_OCTET_STRING_free( issKeyHash );
			return (NULL);
		}

	        if (!(ASN1_OCTET_STRING_set(issKeyHash, md, i))) {
			if( issKeyHash ) ASN1_OCTET_STRING_free( issKeyHash );
			return( NULL );
		}
	}

	/* Build the CERT_IDENTIFIER */
	if((ret = PRQP_cert_id_new(s_name, i_name, serial, cHash, caKeyHash,
					skid, ikid, dgst)) != NULL ) {

		/* Now let's add the certificates to the identifier */
		/*
		if( caCert ) {
			ret->caCert = X509_dup( (X509 *) caCert );
		}

		if( issuedCert ) {
			ret->issuedCert = X509_dup( (X509 *) issuedCert );
		}
		*/
	}

	/* Free data */
	if( cHash ) ASN1_OCTET_STRING_free ( cHash );
	if( caKeyHash ) ASN1_OCTET_STRING_free ( caKeyHash );

	if( skid ) ASN1_OCTET_STRING_free ( skid );
	if( ikid ) ASN1_OCTET_STRING_free ( ikid );
	
	/* return the resulting data structure */
	return( ret );
}


CERT_IDENTIFIER *PRQP_cert_id_new( 
		PKI_X509_NAME *caName, PKI_X509_NAME *caIssuerName,
		ASN1_INTEGER *serial, 
		ASN1_OCTET_STRING *caCertHash, ASN1_OCTET_STRING *caKeyHash,
		ASN1_OCTET_STRING *caKeyId, ASN1_OCTET_STRING *issKeyId,
		PKI_DIGEST_ALG *dgst ) {

        int nid;
        unsigned int i;
        X509_ALGOR *alg;
        CERT_IDENTIFIER *ca_id = NULL;
        unsigned char md[EVP_MAX_MD_SIZE];


	/* To build the Basic Cert Info we need these informations! */
	if( !dgst || !caName ) return (NULL);

        if (!(ca_id = CERT_IDENTIFIER_new())) return (NULL);

	if((ca_id->hashAlgorithm = X509_ALGOR_new()) == NULL ) {
		if(ca_id) CERT_IDENTIFIER_free (ca_id);
		return(NULL);
	}

	alg = ca_id->hashAlgorithm;
	if (alg->algorithm != NULL) ASN1_OBJECT_free(alg->algorithm);
        if ( ((nid = EVP_MD_type(dgst)) == NID_undef) || 
		(!(alg->algorithm=OBJ_nid2obj(nid))) || 
			((alg->parameter=ASN1_TYPE_new()) == NULL)) {

		if(ca_id) CERT_IDENTIFIER_free( ca_id );
		return( NULL );
        }

	alg->parameter->type=V_ASN1_NULL;

	/* Now build the BasicCertIdentifier */

	if((ca_id->basicCertId = BASIC_CERT_IDENTIFIER_new()) == NULL ) {
		if( ca_id ) CERT_IDENTIFIER_free ( ca_id );
		return( NULL );
	}

	if( caIssuerName ) {
		if((ca_id->basicCertId->issuerNameHash = 
					ASN1_OCTET_STRING_new()) == NULL ) {
			if( ca_id ) CERT_IDENTIFIER_free (ca_id);
			return ( NULL );
		}

		if((!X509_NAME_digest(caIssuerName, dgst, md, &i)) || 
			(!(ASN1_OCTET_STRING_set(
				ca_id->basicCertId->issuerNameHash, md, i)))) {
			if(ca_id) CERT_IDENTIFIER_free( ca_id );
			return( NULL );
		}
	}

	/* The caName is already checked - should be present! */
	if((!X509_NAME_digest(caName, dgst, md, &i)) || 
		(!(ASN1_OCTET_STRING_set(ca_id->basicCertId->subjectNameHash,
								md, i)))) {
		if(ca_id) CERT_IDENTIFIER_free( ca_id );
		return( NULL );
	}

	if ((serial) && 
		(!(ca_id->basicCertId->serialNumber = 
						ASN1_INTEGER_dup(serial)))){
		if(ca_id) CERT_IDENTIFIER_free( ca_id );
		return( NULL );
	}

	/* Now build the extInfo structure (if we have enough data!) */

	if( caCertHash ) {
		if((ca_id->extInfo = EXTENDED_CERT_INFO_new()) == NULL ) {
			if( ca_id ) CERT_IDENTIFIER_free (ca_id);
			return( NULL );
		}

		ca_id->extInfo->certificateHash = 
				ASN1_OCTET_STRING_dup ( caCertHash );

		if( caKeyHash ) {
			ca_id->extInfo->subjectKeyHash  =
					ASN1_OCTET_STRING_dup ( caKeyHash );
		}

		if( caKeyId ) {
			ca_id->extInfo->subjectKeyId =
				ASN1_OCTET_STRING_dup ( caKeyId );
		}

		if( issKeyId ) {
			ca_id->extInfo->issuerKeyId =
				ASN1_OCTET_STRING_dup ( issKeyId );
		}

	}

        return ( ca_id );
}

PRQP_REQ *PRQP_REQ_new_cert(PKI_X509_CERT *caCert, 
		PKI_X509_CERT *caIssuerCert, PKI_X509_CERT *issuedCert,
			char *subject_s, char *serial_s, PKI_DIGEST_ALG *md ) {

	CERT_IDENTIFIER *ca_id = NULL;
	RESOURCE_REQUEST_TOKEN *token = NULL;
	BASIC_CONSTRAINTS *bconst = NULL;
	PRQP_REQ *p = NULL;

	if((token = RESOURCE_REQUEST_TOKEN_new()) == NULL ) {
		if( caCert ) PKI_X509_CERT_free ( caCert );
		return(NULL);
	}

	ca_id = PRQP_CERTID_new ( caCert, caIssuerCert, issuedCert, 
						subject_s, serial_s, md );
	if( ca_id == NULL ) {
		if( token ) RESOURCE_REQUEST_TOKEN_free( token );
		return(NULL);
	}

	if((p = PRQP_REQ_new()) == NULL ) {
		if( ca_id ) CERT_IDENTIFIER_free ( ca_id );
		if( token ) RESOURCE_REQUEST_TOKEN_free( token );
		return(NULL);
	}

	if((p->requestData = TBS_REQ_DATA_new()) == NULL ) {
		if( ca_id ) CERT_IDENTIFIER_free ( ca_id );
		if( token ) RESOURCE_REQUEST_TOKEN_free( token );
	}

	if (!ASN1_INTEGER_set(p->requestData->version, 1)) {
		if( ca_id ) CERT_IDENTIFIER_free ( ca_id );
		if( token ) RESOURCE_REQUEST_TOKEN_free( token );
		if( p ) PRQP_REQ_free ( p );
		return(NULL);
	}

	token->ca = ca_id;
	token->resourceList = sk_ASN1_OBJECT_new_null();

	p->requestData->serviceToken = token;
	p->requestData->nonce = PRQP_NONCE_new(80);

        p->requestData->producedAt = (ASN1_GENERALIZEDTIME *) PKI_TIME_new(0);

	return(p);
}

PRQP_REQ *PRQP_REQ_new_url( char * ca_cert_s, char *ca_issuer_cert_s, 
	char *issued_cert_s, char *subject_s, char *serial_s, EVP_MD *md ) {

	PRQP_REQ *p = NULL;
	BIO *cert = NULL;
	PKI_X509_CERT *caCert = NULL;
	PKI_X509_CERT *caIssuerCert = NULL;
	PKI_X509_CERT *issuedCert = NULL;

	URL *cert_url = NULL;
	URL *caCert_url = NULL;

	if( ca_cert_s &&
		((caCert = PKI_X509_CERT_get( ca_cert_s )) == NULL)) {
		return( NULL );
	}

	if( ca_issuer_cert_s &&
		( ( caIssuerCert = PKI_X509_CERT_get (
					(char *)ca_issuer_cert_s))== NULL)){
		if( caCert ) PKI_X509_CERT_free ( caCert );
		return( NULL );
	}

	if( issued_cert_s &&
		(( issuedCert = PKI_X509_CERT_get ( 
					(char *) issued_cert_s) )== NULL) ){
		if( caIssuerCert ) PKI_X509_CERT_free ( caIssuerCert );
		if( caCert ) PKI_X509_CERT_free ( caCert );
		return( NULL );
	}

	if((p = PRQP_REQ_new_cert(caCert, caIssuerCert, issuedCert, 
					subject_s, serial_s, md)) == NULL) {
		if( issuedCert ) PKI_X509_CERT_free ( issuedCert );
		if( caIssuerCert ) PKI_X509_CERT_free ( caIssuerCert );
		if( caCert ) PKI_X509_CERT_free ( caCert );
	}

	return( p );

}


PRQP_REQ *PRQP_REQ_new_file( char *file, int format) {
	PRQP_REQ *p = NULL;
	BIO *req_bio = NULL;

	if ((req_bio=BIO_new(BIO_s_file())) == NULL) {
		return(NULL);
	}

	if (BIO_read_filename(req_bio,file) <= 0) {
		return(NULL);
        }

	if( format == FORMAT_PEM ) {
		p = PEM_read_bio_PRQP_REQ( req_bio );
	} else if ( format == FORMAT_ASN1 ) {
		// return d2i_PRQP_REQ_bio ( req_bio, NULL );
	} else {
		return(NULL);
	}

	return(p);
}

PRQP_REQ * PRQP_REQ_new_certs_res( PKI_X509_CERT *caCert, 
		PKI_X509_CERT *caIssuerCert, PKI_X509_CERT *issuedCert,
		PKI_STACK *sk_services ) {

	PRQP_REQ *p = NULL;

	int i, port, sock;

	p = PRQP_REQ_new_cert( caCert, caIssuerCert, 
					issuedCert, NULL, NULL, NULL );
	if( p == NULL ) {
		PKI_log( PKI_LOG_ERR, "%s::%d::Cannot generate request!\n",
			__FILE__, __LINE__ );
		return(NULL);
	}

	if( !p->requestData || !p->requestData->serviceToken ||
				!p->requestData->serviceToken->resourceList ) {
		PRQP_REQ_free ( p );
		return( NULL );
	}

	for( i = 0; i < PKI_STACK_elements(sk_services); i++ ) {
                char *ss = NULL;
                int  nid = 0;

                PKI_OID *obj = NULL;
		STACK_OF(ASN1_OBJECT) *list = NULL;

		list = p->requestData->serviceToken->resourceList;

                ss = PKI_STACK_get_num( sk_services, i);
                if((obj = PRQP_s2oid( ss )) != NULL ) {
                        sk_ASN1_OBJECT_push( list, obj );
                } else {
                        PKI_log_debug( "%s:%d::ERROR::Can not add %s", 
				__FILE__, __LINE__, ss );
                }
        }

	return( p );
}

int PRQP_REQ_add_service_stack ( PRQP_REQ *p, PKI_STACK *sk_services ) {

	int i = 0;

	if( !p || !sk_services ) return ( PKI_ERR );

	for( i = 0; i < PKI_STACK_elements(sk_services); i++ ) {
		char *ss = NULL;
		int  nid = 0;
		char *ss2=NULL;

		ASN1_OBJECT *obj = NULL;

		ss = PKI_STACK_get_num( sk_services, i);
		if(PRQP_REQ_add_service( p, ss ) == PKI_ERR ) {
			PKI_log( PKI_LOG_INFO, "PRQP REQ, Can not add %s", ss);
		}
		// nid = PRQP_s2id( ss );
		// PRQP_id2s( nid );
	}

	return ( PKI_OK );
}

int PRQP_REQ_add_service ( PRQP_REQ *p, char *ss ) {

	int  nid = 0;

	PKI_OID *obj = NULL;
	STACK_OF(ASN1_OBJECT) *list = NULL;

	if( !p || !ss ) return (NID_undef);

	if( !p->requestData || !p->requestData->serviceToken ||
		!p->requestData->serviceToken->resourceList ) return (PKI_ERR);

	list = p->requestData->serviceToken->resourceList;

	if((obj = PRQP_s2oid( ss )) != NULL ) {
		PKI_log_debug( "Got NID for %s (%d)", __FILE__, __LINE__, 
								ss, nid );
		sk_ASN1_OBJECT_push( list, obj );
	} else {
		PKI_log_debug( "%s:%d::ERROR::Can not add %s", 
						__FILE__, __LINE__, ss );

		if((obj = PKI_OID_new( ss, ss, ss )) == NULL) {
			PKI_log_debug( "ERROR, Can not cread a new obj %s "
					"(%s:%d)", ss, __FILE__, __LINE__ );
			return( PKI_ERR );
		} else {
			sk_ASN1_OBJECT_push( list, obj );
		}
	}

	return( PKI_OK );
}

/*
ASN1_OCTET_STRING *PRQP_NONCE_new (int size) {

	ASN1_OCTET_STRING *nonce = NULL;
	unsigned char *data = NULL;

	if( size < 8 ) size = 8;

	if((data = (unsigned char *) OPENSSL_malloc (size)) == NULL ) {
		return(NULL);
	}
	
	if(!RAND_bytes( data, size)) {
		return (NULL);
	}

	if((nonce = ASN1_OCTET_STRING_new()) == NULL ) {
		if( data ) OPENSSL_free ( data );
		return (NULL);
	}

	ASN1_OCTET_STRING_set( nonce, data, size);

	return( nonce );
}
*/

ASN1_INTEGER *PRQP_NONCE_new(int bits) {
        unsigned char buf[33];
        ASN1_INTEGER *nonce = NULL;
        int len;
        int i;

	if(bits <= 0 ) {
		bits = 80;
	}

	len = (bits - 1) / 8 + 1;

        if (len > (int)sizeof(buf))
		return(NULL);

        if (!RAND_bytes(buf, len))
		return(NULL);

        for (i = 0; i < len && !buf[i]; ++i);
        if (!(nonce = ASN1_INTEGER_new())) return (NULL);
        OPENSSL_free(nonce->data);

        nonce->length = len - i;
        if (!(nonce->data = OPENSSL_malloc(nonce->length + 1)))
		return (NULL);

        memcpy(nonce->data, buf + i, nonce->length);

        return(nonce);

}

// ***************** RESPONSE ******************

int PRQP_RESP_version_set ( PRQP_RESP *resp, int ver ) {

	if( !resp || ! resp->respData ) return (0);

	ASN1_INTEGER_set(resp->respData->version, ver);

	return 1;
}

TBS_RESP_DATA *PRQP_RESP_respdata_get ( PRQP_RESP *resp ) {
	if( !resp ) return ( NULL );

	return( resp->respData );
}

int PRQP_RESP_nonce_dup ( PRQP_RESP *resp, PRQP_REQ *req ) {

	if( !resp || !resp->respData ) return ( PKI_ERR );
	if( !req  || !req->requestData ) return ( PKI_ERR );

	if( req->requestData->nonce != NULL ) {
		resp->respData->nonce = 
			ASN1_OCTET_STRING_dup(req->requestData->nonce);
	}

	return (PKI_OK);
}

int PRQP_RESP_pkistatus_set ( PRQP_RESP *resp, long v, char *info ) {

	if( !resp || !resp->respData ) {
		PKI_log_debug( "Memory Error (missing resp/respData) [%s:%d]",
							__FILE__, __LINE__ );
		return (PKI_ERR);
	}

	/*
	if(resp->respData->pkiStatus == NULL) {
		if((resp->respData->pkiStatus = ASN1_INTEGER_new()) 
								== NULL) {
			PKI_log_debug( "Memory Error (Alloc) [%s:%d]",
							__FILE__, __LINE__ );
			return ( PKI_ERR );
		}
	}
	*/

	ASN1_INTEGER_set( resp->respData->pkiStatus->status, v);

	if( info ) {
		PKI_STRING *str = NULL;

		if((str = PKI_STRING_new_null( PKI_STRING_UTF8 )) == NULL ) {
			PKI_log_debug( "Memory Error %s:%d", __FILE__, __LINE__ );
			return( PKI_ERR );
		}
		if(PKI_STRING_set( str, info, strlen(info)) == PKI_ERR ) {
			PKI_log_debug( "STRING management error at %s:%d",
					__FILE__, __LINE__ );
			PKI_STRING_free ( str );
			return ( PKI_ERR );
		}
		resp->respData->pkiStatus->statusString = str;
	}

	return( PKI_OK );
}

int PRQP_RESP_new_req ( PRQP_RESP **resp_pnt, PRQP_REQ *req, int status,
								long secs ) {

	PRQP_RESP *resp = NULL;
	int i;

	if( *resp_pnt == NULL ) {
		if((resp = PRQP_RESP_new()) == NULL ) {
			return (PKI_ERR);
		}
		*resp_pnt = resp;
	} else {
		resp = *resp_pnt;
	}

	if( resp->respData == NULL ) {
		if((resp->respData = TBS_RESP_DATA_new()) == NULL) {
			PRQP_RESP_free(resp);
			resp = NULL;
			return ( PKI_ERR );
		}
	}

	/*
	if( resp->respData->pkiStatus == NULL ) {
		if((resp->respData->pkiStatus = PKI_STATUS_INFO_new()) 
								== NULL) {
			if( resp ) PRQP_RESP_free( resp );
			return ( PKI_ERR );
		}
	}
	*/

	PRQP_RESP_version_set( resp, 1);

	if( status ) {
		PRQP_RESP_pkistatus_set( resp, status, NULL );
	} else {
		PRQP_RESP_pkistatus_set( resp, 0, NULL );
	}

        resp->respData->producedAt = (ASN1_GENERALIZEDTIME *) PKI_TIME_new(0);

	if( req && req->requestData ) {

		BASIC_CERT_IDENTIFIER * req_caId = NULL;
		BASIC_CERT_IDENTIFIER * resp_caId = NULL;

		/* Let's duplicate the nonce */
		if( req->requestData->nonce ) 
				PRQP_RESP_nonce_dup ( resp, req );

		if( !req->requestData || !req->requestData->serviceToken ||
				!req->requestData->serviceToken->ca ) {

			PRQP_RESP_pkistatus_set( resp, 
					PRQP_STATUS_BAD_REQUEST,
						"Response is Malformed" );

			PKI_log_debug( 
				"PRQP_RESP, error missing fields in REQ!");
		}

		// Let's duplicate the CA Id, maybe it is not needed 
		// as the request/response is tied by the NONCE, but
		// it could be useful for forensics, in case a 
		// response is involved for later verifications

		PKI_log_debug( "Adding caCertId to Response!");

		req_caId = req->requestData->serviceToken->ca->basicCertId;

		// Alloc new memory for the response cert identifier
		resp_caId = BASIC_CERT_IDENTIFIER_new();
		memset( resp_caId, 0, sizeof( BASIC_CERT_IDENTIFIER ));

		// Now let's duplicate the data
		resp_caId->subjectNameHash =
			ASN1_OCTET_STRING_dup ( req_caId->subjectNameHash );

		if( req_caId->serialNumber ) {
			// Theese field is optional 
			resp_caId->serialNumber = 
				ASN1_INTEGER_dup( req_caId->serialNumber );
		}

		if( req_caId->issuerNameHash ) {
			// Theese field is optional
			resp_caId->issuerNameHash =
				ASN1_OCTET_STRING_dup(req_caId->issuerNameHash);
		}

		resp->respData->caCertId = resp_caId;

		//
		// resp->respData->caCertId = (BASIC_CERT_IDENTIFIER *)
		// 	BASIC_CERT_IDENTIFIER_dup( 
		// 			req->requestData->serviceToken->ca->basicCertId);
		/*
		} else {
			PKI_log_debug("Some fields are missing!");
			if( !req->requestData->serviceToken ) {
				PKI_log_debug("Field [serviceToken] is missing in req!");
			} else if( !req->requestData->serviceToken->ca ) {
				PKI_log_debug("Field [serviceToken->ca] is missing in req!");
			} else if( !req->requestData->serviceToken->ca->basicCertId ) {
				PKI_log_debug("Field [serviceToken->ca->basicCertId] is missing in req!");
			}
		}
		*/
	}

	if( secs > 0  ) {
		resp->respData->nextUpdate = PKI_TIME_new ( secs );
	}

	return( PKI_OK );
}


PKI_STACK * PRQP_RESP_url_sk ( PRQP_RESP *r ) {
	int i;
	PKI_STACK *url_sk = NULL;

	if( !r ) return (NULL);

	if((url_sk = PKI_STACK_new_null()) == NULL) {
		return (NULL);
	}

	if( r && r->respData && r->respData->responseToken ) {
		int j;

		STACK_OF(RESOURCE_RESPONSE_TOKEN) * sk_rrt = NULL;
		URL *url = NULL;

		sk_rrt = r->respData->responseToken;
		url_sk = PKI_STACK_new_null();

		for( i = 0; i < sk_RESOURCE_RESPONSE_TOKEN_num( sk_rrt); i++ ) {
			RESOURCE_RESPONSE_TOKEN * rrt = NULL;

			rrt = sk_RESOURCE_RESPONSE_TOKEN_value( sk_rrt, i );
			for( j = 0; j < sk_ASN1_IA5STRING_num ( rrt->resLocatorList ); j++ ) {
				url = URL_new ( i2s_ASN1_IA5STRING( NULL, 
					sk_ASN1_IA5STRING_value ( rrt->resLocatorList, j )));
				if( url ) {
					PKI_STACK_push( url_sk, url );
					// url->type = PRQP_type2d ( rrt->resourceId );
				}
			}
		}
	}

	return ( url_sk );
}

int PRQP_sign( PRQP_REQ *req, PRQP_RESP *resp, PKI_KEYPAIR *pkey, 
		PKI_X509_CERT *x, PKI_DIGEST_ALG *dgst, 
				PKI_X509_CERT_STACK * certs, HSM *hsm) {

	PKI_DIGEST_ALG *dd = NULL;
	BIO *st = NULL;
	int i = 0;

	PRQP_SIGNATURE *psig = NULL;

	if( !pkey || !x ) {
		PKI_log_debug("ERROR:PRQP:Sign key=%p, cert=%p\n", pkey, x );
		PKI_log_debug( "ERROR, missing needed args 2 signing PRQP!");
		return (PKI_ERR);
	}

	if( !dgst )
		dd = (PKI_DIGEST_ALG *) PKI_DIGEST_ALG_SHA1;
	else
		dd = dgst;

	if( req ) {
		if(PRQP_REQ_DATA_sign( req, pkey, dd, hsm) == PKI_ERR ) {
			PKI_log_debug("ERROR, RESP Sign Failed!");
			return(PKI_ERR);
		}
		psig = req->prqpSignature;
	} else if( resp ) {
		if(PRQP_RESP_DATA_sign( resp, pkey, dd, hsm) == PKI_ERR ) {
			PKI_log_debug("ERROR, RESP Sign Failed!");
			return(PKI_ERR);
		}
		psig = resp->prqpSignature;
	} else {
		/* Error!!! */
		return (PKI_ERR);
	}

	if( certs && psig ) {
		if( psig->otherCerts == NULL ) {
			if((psig->otherCerts = sk_X509_new_null()) == NULL) {
				PKI_log_debug("ERROR, Can not Create stack "
						"of certs in signature!");
				return( PKI_ERR );
			}
		}

		for( i = 0; i < PKI_STACK_X509_CERT_elements( certs ); i++ ) {
			sk_X509_push( psig->otherCerts, 
				X509_dup( PKI_STACK_X509_CERT_get_num( 
					certs,  i )));
		}
	}

	PKI_log_debug("INFO, Adding certificate signer's certificate "
								"to RESP!");
	psig->signerCert = X509_dup((X509 *) x);

	return( PKI_OK );
}

int PRQP_RESP_sign_tk(PRQP_RESP *resp, PKI_TOKEN *tk, PKI_DIGEST_ALG *dgst ){

	if( !tk || !tk->keypair || !tk->cert ) return ( PKI_ERR );
	
	return ( PRQP_sign( NULL, resp, tk->keypair, tk->cert, dgst, 
						tk->otherCerts, tk->hsm ));
}

int PRQP_RESP_sign ( PRQP_RESP *resp, PKI_KEYPAIR *pkey, PKI_X509_CERT *x,
		PKI_DIGEST_ALG *dgst, PKI_X509_CERT_STACK * certs, HSM *hsm ) {

	return ( PRQP_sign( NULL, resp, pkey, x, dgst, certs, hsm ));

}

int PRQP_RESP_is_signed( PRQP_RESP *resp ) {

	if( !resp || !resp->prqpSignature ) {
		return ( PKI_ERR );
	}

	return( PKI_OK );
}

int PRQP_RESP_verify ( PRQP_RESP *r ) {

	PKI_KEYPAIR *key = NULL;
	PKI_X509_CERT *x = NULL;

	if( PRQP_RESP_is_signed (r) == PKI_ERR ) {
		PKI_log_debug("PRQP_RESP_verify() - ERR, resp is not signed!");
		return ( PKI_ERR );
	}

	if( (x = PRQP_RESP_get_data(r, PRQP_DATATYPE_SIGNER_CERT )) == NULL ) {
		PKI_log_debug("PRQP_RESP_verify() - ERR, no cert in response!");
		return ( PKI_ERR );
	}

	if((key = PKI_X509_CERT_get_data( x, PKI_X509_KEYPAIR )) == NULL){
		PKI_log_debug("PRQP_RESP_verify() - ERR, no keypair in sgner's cert!");
		return (PKI_ERR);
	}

	return ( PKI_verify( PKI_OBJTYPE_PRQP_RESP, r, key, NULL ));
}

/* SIGNATURES --- PRQP Responses */
/*
#define PRQP_RESP_DATA_sign(o,pkey,md,d) \
        ASN1_item_sign(ASN1_ITEM_rptr(TBS_RESP_DATA), \
		o->signature->signatureAlgorithm,NULL,\
                o->signature->signature,o->respData,pkey,md)
*/

int PRQP_RESP_DATA_sign (PRQP_RESP *resp, PKI_KEYPAIR *pkey, 
					PKI_DIGEST_ALG *md, HSM *hsm ) {

	int ret = 0;
	EVP_PKEY *pp = NULL;

	if( !resp || !pkey ) return (PKI_ERR);

	if( !md ) md = PKI_DIGEST_ALG_SHA1;

	if(resp->prqpSignature == NULL ) {
		resp->prqpSignature = PRQP_SIGNATURE_new();
		if( !resp->prqpSignature ) {
			PKI_log( PKI_LOG_ERR, "Memory Allocation Err (%s:%d)",
				__FILE__, __LINE__ );
			return( PKI_ERR );
		}
	}

	/*
	if( resp->prqpSignature->otherCerts == NULL ) {
		PKI_log_debug("PRQP_RESP_DATA_sign() - Adding certs stack!");
		resp->prqpSignature->otherCerts = sk_X509_new( NULL );
		if( !resp->prqpSignature->otherCerts ) {
			PKI_log( PKI_LOG_ERR, "Memory Allocation Err (%s:%d)",
				__FILE__, __LINE__ );
			return( PKI_ERR );
		}
	}
	*/

	pp = (EVP_PKEY *) pkey;

	ret = PKI_ITEM_sign( PKI_OBJTYPE_UNKNOWN, 
			resp->respData, TBS_RESP_DATA,
			resp->prqpSignature->signatureAlgorithm,
			resp->prqpSignature->signature, pkey, md, hsm );

	if( ret == PKI_ERR ) {
		PRQP_SIGNATURE_free ( resp->prqpSignature );
		PKI_log( PKI_LOG_ERR, "PRQP_RESP_DATA_sign() - ERROR : %s",
			ERR_error_string(ERR_get_error(), NULL ));
		resp->prqpSignature = NULL;
		return( PKI_ERR );
	}

	return ( PKI_OK );
}

/*
#define PRQP_RESP_DATA_verify2(a,r,d) \
	ASN1_item_verify(ASN1_ITEM_rptr(TBS_RESP_DATA),\
        	a->signature->signatureAlgorithm,a->signature->signature, \
		a->respData,r)
*/

/*
int PRQP_RESP_DATA_verify ( PRQP_RESP *resp, PKI_KEYPAIR *pkey) {

	int ret = 0;
	EVP_PKEY *pp = NULL;

	if( !resp || !pkey ) {
		PKI_log_debug("RESP_DATA_verify() - Missing resp or key!");
		return (PKI_ERR);
	}

	return ( PKI_verify( PKI_OBJTYPE_PRQP_RESP, resp, pkey, NULL ));

	pp = (EVP_PKEY *) pkey;

	ERR_clear_error();

	ret = ASN1_item_verify(ASN1_ITEM_rptr(TBS_RESP_DATA),
		resp->prqpSignature->signatureAlgorithm,
		resp->prqpSignature->signature, resp->respData, pp );

	if( ret == 0 ) {
		PKI_log_debug( "RESP_DATA_verify() - ERROR:%s",
			ERR_error_string(ERR_get_error(), NULL ));
		return ( PKI_ERR );
	}

	PKI_log_debug( "RESP_DATA_verify() - OK");

	return ( PKI_OK );
}
*/

/* SIGNATURES --- PRQP Requests*/
/*
#define PRQP_REQ_DATA_sign(o,pkey,md,d) \
        ASN1_item_sign(ASN1_ITEM_rptr(TBS_REQ_DATA), \
		o->signature->signatureAlgorithm,NULL,\
                o->signature->signature,o->requestData,pkey,md)
*/

int PRQP_REQ_sign_tk( PRQP_REQ *req, PKI_TOKEN *tk, PKI_DIGEST_ALG *dgst ){

	if( !tk || !tk->keypair || !tk->cert ) return ( PKI_ERR );
	
	return ( PRQP_sign( req, NULL, tk->keypair, tk->cert, dgst, 
						tk->otherCerts, tk->hsm ));
}

int PRQP_REQ_sign( PRQP_REQ *req, PKI_KEYPAIR *pkey, PKI_X509_CERT *x,
		PKI_DIGEST_ALG *dgst, PKI_X509_CERT_STACK * certs, HSM *hsm ){

	return ( PRQP_sign( req, NULL, pkey, x, dgst, certs, hsm ));
}

int PRQP_REQ_DATA_sign (PRQP_REQ *req, PKI_KEYPAIR *pkey, 
					PKI_DIGEST_ALG *md, HSM *hsm) {

	int ret = 0;
	EVP_PKEY *pp = NULL;

	if( !req || !pkey ) return (PKI_ERR);

	if( !md ) md = PKI_DIGEST_ALG_SHA1;

	if(req->prqpSignature == NULL ) {
		req->prqpSignature = PRQP_SIGNATURE_new();
		if( !req->prqpSignature ) {
			PKI_log( PKI_LOG_ERR, "Memory Allocation Err (%s:%d)",
				__FILE__, __LINE__ );
			return( PKI_ERR );
		}
	}

	/*
	if( req->prqpSignature->otherCerts == NULL ) {
		PKI_log_debug("REQ_DATA_sign() - Adding other certs stack!");
		req->prqpSignature->otherCerts = sk_X509_new( NULL );
		if( !req->prqpSignature->otherCerts ) {
			PKI_log( PKI_LOG_ERR, "Memory Allocation Err (%s:%d)",
				__FILE__, __LINE__ );
			return( PKI_ERR );
		}
	}
	*/

	ERR_clear_error();

	pp = (EVP_PKEY *) pkey;
	ret = PKI_ITEM_sign( PKI_OBJTYPE_UNKNOWN, 
			req->requestData, TBS_REQ_DATA,
			req->prqpSignature->signatureAlgorithm,
			req->prqpSignature->signature, pkey, md, hsm );

	if( ret == PKI_ERR ) {
		PRQP_SIGNATURE_free ( req->prqpSignature );
		PKI_log( PKI_LOG_ERR, "PRQP_REQ_DATA_sign() - ERROR:%s",
			ERR_error_string(ERR_get_error(), NULL ));
		return( PKI_ERR );
	}

	return ( PKI_OK );

}

/*
#define PRQP_REQ_DATA_verify(a,r,d) \
	ASN1_item_verify(ASN1_ITEM_rptr(TBS_REQ_DATA),\
        	a->signature->signatureAlgorithm,a->signature->signature, \
		a->requestData,r)
*/

int PRQP_REQ_is_signed( PRQP_REQ *r ) {

	if( !r || !r->prqpSignature ) return ( PKI_ERR );

	return( PKI_OK );
}


int PRQP_REQ_verify ( PRQP_REQ *r ) {

	PKI_KEYPAIR *key = NULL;
	PKI_X509_CERT *x = NULL;

	if( PRQP_REQ_is_signed (r) == PKI_ERR ) {
		PKI_log_debug( "PRQP_REQ_verify() - ERR, req not signed!");
		return ( PKI_ERR );
	}

	if( (x = PRQP_REQ_get_data(r, PRQP_DATATYPE_SIGNER_CERT )) == NULL ) {
		PKI_log_debug( "PRQP_REQ_verify() - ERR, no cert in req!");
		return ( PKI_ERR );
	}

	if((key = PKI_X509_CERT_get_data( x, PKI_X509_KEYPAIR )) == NULL){
		PKI_log_debug( "PRQP_REQ_verify() - ERR, no pubkey found!");
		return (PKI_ERR);
	}

	return ( PKI_verify( PKI_OBJTYPE_PRQP_REQ, r, key, NULL ));
}

/*
int PRQP_REQ_DATA_verify ( PRQP_REQ *req, PKI_KEYPAIR *pkey) {

	int ret = 0;
	EVP_PKEY *pp = NULL;

	if( !req || !pkey ) {
		PKI_log_debug("REQ_DATA_verify() - Missing req or key!");
		return (PKI_ERR);
	}

	ERR_clear_error();

	pp = (EVP_PKEY *) pkey;

	ret = ASN1_item_verify(ASN1_ITEM_rptr(TBS_REQ_DATA),
		req->prqpSignature->signatureAlgorithm,
		req->prqpSignature->signature, req->requestData, pp );

	if( ret == 0 ) {
		PKI_log_debug( "PRQP_REQ_verify() - ERROR: %s",
			ERR_error_string(ERR_get_error(), NULL ));
		return( PKI_ERR );
	}

	PKI_log_debug( "REQ_DATA_verify() - OK");

	return ( PKI_OK );
}
*/

// ===================== Print Functions for REQ/RESP =================

int PRQP_REQ_print ( PRQP_REQ *req ) {
	return (PRQP_REQ_print_fp( stdout, req ));
}

int PRQP_REQ_print_fp ( FILE *fp, PRQP_REQ *req ) {

	TBS_REQ_DATA *rd = NULL;
	BASIC_CERT_IDENTIFIER *ci = NULL;
	RESOURCE_REQUEST_TOKEN *rt = NULL;
	PKI_TIME *time = NULL;
	int i = 0;

	if( !req || !req->requestData ) return (PKI_ERR);

	rd = req->requestData;

	fprintf( fp, "PRQP Request:\r\n");
	fprintf( fp, "    Version: %s (0x%s)\r\n", 
			i2s_ASN1_INTEGER(NULL, rd->version ),
			i2s_ASN1_INTEGER(NULL, rd->version ));

	if( rd->nonce ) {
		fprintf( fp, "    Nonce: %s\r\n", 
			i2s_ASN1_OCTET_STRING(NULL, rd->nonce));
				// i2s_ASN1_INTEGER( NULL, rd->nonce ));
	} else {
		fprintf( fp, "    Nonce: %s\r\n", "[ Not Present ]" );
	}

	if((time = PRQP_REQ_get_data( req, PRQP_DATATYPE_PRODUCEDAT )) 
								!= NULL ) {
		fprintf( fp, "    Produced At: ");
		PKI_TIME_print_fp( fp, time );
		fprintf( fp, "\r\n");
	}

	ci = rd->serviceToken->ca->basicCertId;

	fprintf( fp, "\r\n");
	// fprintf( fp, "    Resources Identifier:\r\n" );
	fprintf( fp, "    Certification Authority Identifier:\r\n" );

	if( ci->serialNumber ) {
		fprintf( fp, "        Serial Number:\r\n");
		fprintf( fp, "            %s (0x%s)\r\n", 
			i2s_ASN1_INTEGER(NULL,ci->serialNumber),
			i2s_ASN1_OCTET_STRING(NULL,ci->serialNumber));
	} else {
		fprintf( fp, "        Serial Number: %s\r\n", 
			"[ Not Present ]" );
	}

	if( ci->subjectNameHash ) {
		fprintf( fp, "        Subject Name Hash:\r\n");
		fprintf( fp, "            %s\r\n", 
			i2s_ASN1_OCTET_STRING( NULL, ci->subjectNameHash) );
	} else {
		fprintf( fp, "         Subject Name Hash:\r\n");
		fprintf( fp, "            %s\r\n", 
			"[ Not Present ]" );
	}

	if( ci->issuerNameHash ) {
		fprintf( fp, "        Issuer Name Hash:\r\n");
		fprintf( fp, "            %s\r\n", 
			i2s_ASN1_OCTET_STRING(NULL, ci->issuerNameHash));
	}

	fprintf( fp, "\r\n    Requested Services:\r\n        " );
	rt = req->requestData->serviceToken;
	if( (rt == NULL) || ( rt->resourceList == NULL ) || 
			(sk_ASN1_OBJECT_num( rt->resourceList ) < 1) ) {
		fprintf( fp, "ALL\r\n");
	} else {
		for( i=0; i < sk_ASN1_OBJECT_num( rt->resourceList ); i++ ) {
                        URL *url = NULL;
			ASN1_OBJECT *ri = NULL;

			ri = sk_ASN1_OBJECT_value(rt->resourceList,i);
                        // url = PRQP_STACK_get_num( rt->serviceList, i );
			if( i > 0 ) fprintf( fp, ", ");

                        fprintf( fp, "%s", (char *) PRQP_oid2s(ri) );
               }
	}
	fprintf( fp, "\r\n" );

	if( rd->extensions ) {
		fprintf( fp, "   Extensions:\r\n");
		fprintf( fp, "      *** EXTENSIONS PRESENT ***\r\n");
	}

	if( req->prqpSignature && req->prqpSignature->signature ) {
		BIO * out = NULL;

		if((out = BIO_new_fp( fp, BIO_NOCLOSE )) != NULL ) {
			X509_signature_print( out, 
				req->prqpSignature->signatureAlgorithm, 
					req->prqpSignature->signature );
			BIO_free ( out );
		}
		fprintf(fp, "    Signature Verification: ");
		if(PRQP_REQ_verify( req ) == PKI_OK ) {
			fprintf( fp, "Ok.\r\n" );
		} else {
			fprintf( fp, "Error => ");
			ERR_print_errors_fp( fp );
			fprintf(fp, "\r\n");
		}
	}

	return (PKI_OK);
}

int PRQP_RESP_print ( PRQP_RESP *resp ) {
	return (PRQP_RESP_print_fp( stdout, resp ));
}

int PRQP_RESP_print_fp ( FILE *fp, PRQP_RESP *resp ) {

	TBS_RESP_DATA *rd = NULL;
	BASIC_CERT_IDENTIFIER *ci = NULL;
	RESOURCE_RESPONSE_TOKEN *rt = NULL;
	RESOURCE_RESPONSE_TOKEN *rr_sk = NULL;
	PKI_RESOURCE_RESPONSE_TOKEN_STACK *pki_sk=NULL;

	char *time_s = NULL;
	PKI_TIME *time = NULL;

	int i = 0;
	int status = 0;

	if( !resp || !resp->respData ) return (PKI_ERR);

	rd = resp->respData;

	fprintf( fp, "PRQP Response:\r\n");
	fprintf( fp, "    Version: %s (0x%s)\r\n", 
			i2s_ASN1_INTEGER(NULL, rd->version ),
			i2s_ASN1_INTEGER(NULL, rd->version ));

	if( rd->nonce ) {
		fprintf( fp, "    Nonce: %s\r\n", 
			i2s_ASN1_OCTET_STRING(NULL, rd->nonce));
				// i2s_ASN1_INTEGER( NULL, rd->nonce ));
	} else {
		fprintf( fp, "    Nonce: %s\r\n", "[ Not Present ]" );
	}

	if((time = PRQP_RESP_get_data( resp, PRQP_DATATYPE_PRODUCEDAT )) 
								!= NULL ) {
		fprintf( fp, "    Produced At: ");
		PKI_TIME_print_fp( fp, time );
		fprintf( fp, "\r\n");
	}

	if((time = PRQP_RESP_get_data( resp, PRQP_DATATYPE_NEXTUPDATE )) 
								!= NULL ) {

		fprintf( fp, "    Next Update: ");
		PKI_TIME_print_fp( fp, time );
		fprintf( fp, "\r\n");
	}

	fprintf( fp, "    PKI Status: %s\r\n", 
			PRQP_RESP_get_data( resp, 
				PRQP_DATATYPE_PKI_STATUS_INFO_VALUE ));

	ci = rd->caCertId;

	fprintf( fp, "\r\n    Certification Authority Identifier:\r\n" );
	// fprintf( fp, "      Certification Authority:\r\n");
	if( ci->serialNumber ) {
		fprintf( fp, "        Serial Number:\r\n");
		fprintf( fp, "            %s (0x%s)\r\n", 
			i2s_ASN1_INTEGER(NULL,ci->serialNumber),
			i2s_ASN1_OCTET_STRING(NULL,ci->serialNumber));
	} else {
		fprintf( fp, "        Serial Number: %s\r\n", 
			"[ Not Present ]" );
	}

	if( ci->subjectNameHash ) {
		fprintf( fp, "        Subject Name Hash:\r\n");
		fprintf( fp, "            %s\r\n", 
			i2s_ASN1_OCTET_STRING( NULL, ci->subjectNameHash) );
	} else {
		fprintf( fp, "        Subject Name Hash:\r\n");
		fprintf( fp, "            %s\r\n", 
			"[ Not Present ]" );
	}

	if( ci->issuerNameHash ) {
		fprintf( fp, "        Issuer Name Hash:\r\n");
		fprintf( fp, "            %s\r\n", 
			i2s_ASN1_OCTET_STRING(NULL, ci->issuerNameHash));
	}

	fprintf( fp, "\r\n");
	if( PRQP_RESP_get_data( resp, PRQP_DATATYPE_PKI_STATUS_INFO_VALUE ) 
						== PRQP_STATUS_STRING_OK ) {
	   fprintf( fp, "    Requested Services:\r\n");
	   if(( pki_sk=PRQP_RESP_get_data(resp,PRQP_DATATYPE_SERVICES))!= NULL){

		RESOURCE_RESPONSE_TOKEN *res = NULL;

		while( res = PKI_STACK_RESOURCE_RESPONSE_TOKEN_pop( pki_sk )) {
			PKI_STRING *str = NULL;
			fprintf( fp, "        %s:\r\n", 
					PRQP_oid2s( res->resourceId ));

			while((str = sk_ASN1_IA5STRING_pop( 
					res->resLocatorList )) != NULL) {;

				fprintf( fp, "            ");
				PKI_STRING_print_fp( fp, str );
				fprintf( fp, "\r\n");
				// PKI_STRING_free( str );
			}
		}
		if( res ) RESOURCE_RESPONSE_TOKEN_free ( res );
	   } else {
		PKI_log_debug("Parsing Response, no SERVICES found!");
	   }
	}

	fprintf( fp, "\r\n" );

	if( rd->extensions ) {
		fprintf( fp, "   Extensions:\r\n");
		fprintf( fp, "      *** EXTENSIONS PRESENT ***\r\n");
	}

	if( resp->prqpSignature && resp->prqpSignature->signature ) {
		BIO * out = NULL;

		if((out = BIO_new_fp( fp, BIO_NOCLOSE )) != NULL ) {
			X509_signature_print( out, 
				resp->prqpSignature->signatureAlgorithm, 
					resp->prqpSignature->signature );
			BIO_free ( out );
		}

		fprintf(fp, "    Signature Verification: ");
		if(PRQP_RESP_verify( resp ) == PKI_OK ) {
			fprintf( fp, "Ok\r\n" );
		} else {
			fprintf( fp, "Error\r\n");
		}
	}

	return (PKI_OK);
}

void * PRQP_REQ_get_data ( PRQP_REQ *r, PRQP_DATATYPE type ) {

	void * ret = NULL;

	if( !r ) return ( NULL );

	switch ( type ) {
		case PRQP_DATATYPE_VERSION:
			ret = r->requestData->version;
			break;
		case PRQP_DATATYPE_NONCE:
			ret = r->requestData->nonce;
			break;
		case PRQP_DATATYPE_PRODUCEDAT:
			ret = r->requestData->producedAt;
			break;
		case PRQP_DATATYPE_SIGNER_CERT:
			if( r->prqpSignature ) {
				ret = r->prqpSignature->signerCert;
			}
			break;
		case PRQP_DATATYPE_OTHER_CERTS:
			if( r->prqpSignature && r->prqpSignature->otherCerts) {
				int i = 0;
				PKI_X509_CERT_STACK *s = NULL;

				s = PKI_STACK_X509_CERT_new();
				for( i = 0 ; i < sk_X509_num( 
						r->prqpSignature->otherCerts );
								 i++ ) {

					PKI_STACK_X509_CERT_push( ret, 
						sk_X509_value( 
						  r->prqpSignature->otherCerts,
							 i));
				}
				ret = s;
			}
			break;
		case PRQP_DATATYPE_SIGNATURE:
			if( r->prqpSignature ) {
				ret = r->prqpSignature->signature;
			}
			break;
		case PRQP_DATATYPE_SERVICES:
			if( r->requestData && 
				r->requestData->serviceToken  &&
				  r->requestData->serviceToken->resourceList ) {

				STACK_OF(ASN1_OBJECT) *objs = NULL;
				PKI_OID_STACK *sk = NULL;
				int i = 0;

				PKI_log_debug("get_data() - Request has a resourceList");

				sk = PKI_STACK_OID_new_null();

				objs = r->requestData->serviceToken->resourceList;
				PKI_log_debug("get_data() - Number of OIDs in request is %d", sk_ASN1_OBJECT_num( objs ) );
				for( i=0; i< sk_ASN1_OBJECT_num (objs); i++ ) {
					ASN1_OBJECT *p = NULL;

					p = sk_ASN1_OBJECT_value( objs, i);
					PKI_STACK_OID_push( sk, p );
				}
				ret = sk;
			} else {
				PKI_log_debug( "get_data() - No resources in request");
				PKI_log_debug( "get_data() - r->requestData %p",
							r->requestData );
				PKI_log_debug( "get_data() - "
					"r->requestData->serviceToken %p",
						r->requestData->serviceToken );
				PKI_log_debug( "get_data() - "
					"r->requestData->serviceToken->resourceList %p",
					r->requestData->serviceToken->resourceList );
			}
			break;
		default:
			/* Datatype not supported */
			return ( NULL );
	}

	return ( ret );
}

void * PRQP_RESP_get_data ( PRQP_RESP *r, PRQP_DATATYPE type ) {

	void *ret = NULL;

	if( !r ) return ( NULL );

	switch ( type ) {
		case PRQP_DATATYPE_VERSION:
			ret = r->respData->version;
			break;
		case PRQP_DATATYPE_NONCE:
			ret = r->respData->nonce;
			break;
		case PRQP_DATATYPE_PRODUCEDAT:
			ret = r->respData->producedAt;
			break;
		case PRQP_DATATYPE_NEXTUPDATE:
			ret = r->respData->nextUpdate;
			break;
		case PRQP_DATATYPE_SIGNER_CERT:
			if( r->prqpSignature ) {
				ret = r->prqpSignature->signerCert;
			}
			break;
		case PRQP_DATATYPE_PKI_STATUS_INFO_VALUE:
			if( r->respData && r->respData->pkiStatus ) {
				ASN1_INTEGER *tmp_int = NULL;
				long a = 0;

				tmp_int = r->respData->pkiStatus->status;
				ret = i2s_ASN1_INTEGER( NULL, tmp_int );
				if( ret ) {
					a = atol( ret );
					PKI_Free( ret );

					if( a < PRQP_STATUS_STRING_NUM ) {
						ret=PRQP_STATUS_STRING[a];
					} else {
						ret=PRQP_STATUS_STRING_UNKNOWN;
					}
				} else {
					ret = PRQP_STATUS_STRING_UNKNOWN;
				}
			};
			break;
		case PRQP_DATATYPE_PKI_STATUS_INFO_STRING:
			if( r->respData && r->respData->pkiStatus ) {
				PKI_STRING *s = NULL;
				s = r->respData->pkiStatus->statusString;
				ret = PKI_STRING_get_utf8( s );
			};
			break;
		case PRQP_DATATYPE_OTHER_CERTS:
			if( r->prqpSignature && r->prqpSignature->otherCerts) {
				int i = 0;
				PKI_X509_CERT_STACK *s = NULL;

				s = PKI_STACK_X509_CERT_new();
				for( i = 0 ; i < sk_X509_num( 
						r->prqpSignature->otherCerts );
								 i++ ) {

					PKI_STACK_X509_CERT_push( ret, 
						sk_X509_value( 
						  r->prqpSignature->otherCerts,
							 i));
				}
				ret = s;
			}
			break;
		case PRQP_DATATYPE_SIGNATURE:
			if( r->prqpSignature ) {
				ret = r->prqpSignature->signature;
			}
			break;
		case PRQP_DATATYPE_SERVICES:
			if( r->respData && 
				r->respData->responseToken ) {

				int i = 0;
				int num = 0;

				STACK_OF(RESOURCE_RESPONSE_TOKEN) *rrt = NULL;
				STACK_OF(ASN1_OBJECT) *objs = NULL;
				PKI_OID_STACK *sk = NULL;
				PKI_RESOURCE_RESPONSE_TOKEN_STACK *pki_sk=NULL;

				pki_sk = PKI_STACK_RESOURCE_RESPONSE_TOKEN_new_null();
				if( !pki_sk ) return ( NULL );

				rrt = r->respData->responseToken;
 
				num = sk_RESOURCE_RESPONSE_TOKEN_num ( rrt );
				for( i=0; i < num; i++ ) {
					RESOURCE_RESPONSE_TOKEN *p = NULL;

					p = sk_RESOURCE_RESPONSE_TOKEN_value( 
								rrt, i);
					PKI_STACK_RESOURCE_RESPONSE_TOKEN_push(
						 pki_sk, RESOURCE_RESPONSE_TOKEN_dup( p ) );
				}
				ret = pki_sk;
			} else {
				if( !r->respData ) {
					PKI_log_debug("PRQP RESP:Missing r->respData [get_data]");
				} else if ( !r->respData->responseToken ) {
					PKI_log_debug("PRQP RESP:Missing r->respData->responseToken [get_data]");
				}
			};
			break;
		default:
			/* Datatype not supported */
			return ( NULL );
	}

	return ( ret );
}

PKI_OID *PRQP_RESOURCE_RESPONSE_TOKEN_get_oid ( RESOURCE_RESPONSE_TOKEN *rrt ) {

	if( !rrt ) return ( NULL );

	return( (PKI_OID *) rrt->resourceId );
}

PKI_STACK *PRQP_RESOURCE_RESPONSE_TOKEN_get_services( RESOURCE_RESPONSE_TOKEN *rrt ) {

	int i = 0;
	PKI_STACK *ret = NULL;

	if ( !rrt || !rrt->resourceId ) return (NULL);

	if((ret = PKI_STACK_new( NULL )) == NULL ) {
		PKI_log_debug( "Memory Allocation Failed at %s:%d",
						__FILE__, __LINE__ );
		return ( NULL );
	}

	for(i=0; i< sk_ASN1_IA5STRING_num( rrt->resLocatorList ); i++ ) {
		ASN1_IA5STRING *s = NULL;
		char *ret_s = NULL;

		s = sk_ASN1_IA5STRING_value( rrt->resLocatorList, i );
		ret_s = i2s_ASN1_IA5STRING( NULL, s );
		PKI_STACK_push( ret, s );
	}

	return( ret );
}

/*
int PRQP_PKI_STATUS_INFO_get_status ( PRQP_RESP *r ) {

	ASN1_INTEGER *tmp_int = NULL;
	char *tmp_s = NULL;

	long a = 0;
	int ret = PRQP_STATUS_UNKNOWN;

	if( !r || !r->respData || !r->respData->pkiStatus ) {
		return ( PRQP_STATUS_UNKNOWN );
	}

	tmp_int = r->respData->pkiStatus;
	tmp_s = i2s_ASN1_INTEGER( NULL, tmp_int );
				
	if( tmp_s ) {
		a = atol( tmp_s );
		PKI_Free( tmp_s );

		if( a < PRQP_STATUS_STRING_NUM ) {
			ret=a;
		} else {
			ret=PRQP_STATUS_UNKNOWN;
		}
	}

	return ( ret );
}
*/

